# Copyright 2011 OpenStack Foundation
# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
from unittest import mock

import glance_store
from oslo_config import cfg
import oslo_messaging
import webob

import glance.async_
from glance.common import exception
from glance.common import timeutils
import glance.context
from glance import notifier
import glance.tests.unit.utils as unit_test_utils
from glance.tests import utils


DATETIME = datetime.datetime(2012, 5, 16, 15, 27, 36, 325355)


UUID1 = 'c80a1a6c-bd1f-41c5-90ee-81afedb1d58d'
USER1 = '54492ba0-f4df-4e4e-be62-27f4d76b29cf'
TENANT1 = '6838eb7b-6ded-434a-882c-b344c77fe8df'
TENANT2 = '2c014f32-55eb-467d-8fcb-4bd706012f81'


class ImageStub(glance.domain.Image):
    def get_data(self, offset=0, chunk_size=None):
        return ['01234', '56789']

    def set_data(self, data, size, backend=None, set_active=True):
        for chunk in data:
            pass


class ImageRepoStub(object):
    def remove(self, *args, **kwargs):
        return 'image_from_get'

    def save(self, *args, **kwargs):
        return 'image_from_save'

    def add(self, *args, **kwargs):
        return 'image_from_add'

    def get(self, *args, **kwargs):
        return 'image_from_get'

    def list(self, *args, **kwargs):
        return ['images_from_list']


class ImageMemberRepoStub(object):
    def remove(self, *args, **kwargs):
        return 'image_member_from_remove'

    def save(self, *args, **kwargs):
        return 'image_member_from_save'

    def add(self, *args, **kwargs):
        return 'image_member_from_add'

    def get(self, *args, **kwargs):
        return 'image_member_from_get'

    def list(self, *args, **kwargs):
        return ['image_members_from_list']


class TaskStub(glance.domain.TaskStub):
    def run(self, executor):
        pass


class Task(glance.domain.Task):
    def succeed(self, result):
        pass

    def fail(self, message):
        pass


class TaskRepoStub(object):
    def remove(self, *args, **kwargs):
        return 'task_from_remove'

    def save(self, *args, **kwargs):
        return 'task_from_save'

    def add(self, *args, **kwargs):
        return 'task_from_add'

    def get_task(self, *args, **kwargs):
        return 'task_from_get'

    def list(self, *args, **kwargs):
        return ['tasks_from_list']


class TestNotifier(utils.BaseTestCase):

    @mock.patch.object(oslo_messaging, 'Notifier')
    @mock.patch.object(oslo_messaging, 'get_notification_transport')
    def _test_load_strategy(self,
                            mock_get_transport, mock_notifier,
                            url, driver):
        nfier = notifier.Notifier()
        mock_get_transport.assert_called_with(cfg.CONF)
        self.assertIsNotNone(nfier._transport)
        mock_notifier.assert_called_with(nfier._transport,
                                         publisher_id='image.localhost')
        self.assertIsNotNone(nfier._notifier)

    def test_notifier_load(self):
        self._test_load_strategy(url=None, driver=None)

    @mock.patch.object(oslo_messaging, 'set_transport_defaults')
    def test_set_defaults(self, mock_set_trans_defaults):
        notifier.set_defaults(control_exchange='foo')
        mock_set_trans_defaults.assert_called_with('foo')
        notifier.set_defaults()
        mock_set_trans_defaults.assert_called_with('glance')


class TestImageNotifications(utils.BaseTestCase):
    """Test Image Notifications work"""

    def setUp(self):
        super(TestImageNotifications, self).setUp()
        self.image = ImageStub(
            image_id=UUID1, name='image-1', status='active', size=1024,
            created_at=DATETIME, updated_at=DATETIME, owner=TENANT1,
            visibility='public', container_format='ami', virtual_size=2048,
            tags=['one', 'two'], disk_format='ami', min_ram=128,
            min_disk=10, checksum='ca425b88f047ce8ec45ee90e813ada91',
            locations=['http://127.0.0.1'])
        self.context = glance.context.RequestContext(tenant=TENANT2,
                                                     user=USER1)
        self.image_repo_stub = ImageRepoStub()
        self.notifier = unit_test_utils.FakeNotifier()
        self.image_repo_proxy = glance.notifier.ImageRepoProxy(
            self.image_repo_stub, self.context, self.notifier)
        self.image_proxy = glance.notifier.ImageProxy(
            self.image, self.context, self.notifier)

    def test_image_save_notification(self):
        self.image_repo_proxy.save(self.image_proxy)
        output_logs = self.notifier.get_logs()
        self.assertEqual(1, len(output_logs))
        output_log = output_logs[0]
        self.assertEqual('INFO', output_log['notification_type'])
        self.assertEqual('image.update', output_log['event_type'])
        self.assertEqual(self.image.image_id, output_log['payload']['id'])
        if 'location' in output_log['payload']:
            self.fail('Notification contained location field.')

    def test_image_save_notification_disabled(self):
        self.config(disabled_notifications=["image.update"])
        self.image_repo_proxy.save(self.image_proxy)
        output_logs = self.notifier.get_logs()
        self.assertEqual(0, len(output_logs))

    def test_image_add_notification(self):
        self.image_repo_proxy.add(self.image_proxy)
        output_logs = self.notifier.get_logs()
        self.assertEqual(1, len(output_logs))
        output_log = output_logs[0]
        self.assertEqual('INFO', output_log['notification_type'])
        self.assertEqual('image.create', output_log['event_type'])
        self.assertEqual(self.image.image_id, output_log['payload']['id'])
        if 'location' in output_log['payload']:
            self.fail('Notification contained location field.')

    def test_image_add_notification_disabled(self):
        self.config(disabled_notifications=["image.create"])
        self.image_repo_proxy.add(self.image_proxy)
        output_logs = self.notifier.get_logs()
        self.assertEqual(0, len(output_logs))

    def test_image_delete_notification(self):
        self.image_repo_proxy.remove(self.image_proxy)
        output_logs = self.notifier.get_logs()
        self.assertEqual(1, len(output_logs))
        output_log = output_logs[0]
        self.assertEqual('INFO', output_log['notification_type'])
        self.assertEqual('image.delete', output_log['event_type'])
        self.assertEqual(self.image.image_id, output_log['payload']['id'])
        self.assertTrue(output_log['payload']['deleted'])
        if 'location' in output_log['payload']:
            self.fail('Notification contained location field.')

    def test_image_delete_notification_disabled(self):
        self.config(disabled_notifications=['image.delete'])
        self.image_repo_proxy.remove(self.image_proxy)
        output_logs = self.notifier.get_logs()
        self.assertEqual(0, len(output_logs))

    def test_image_get(self):
        image = self.image_repo_proxy.get(UUID1)
        self.assertIsInstance(image, glance.notifier.ImageProxy)
        self.assertEqual('image_from_get', image.repo)

    def test_image_list(self):
        images = self.image_repo_proxy.list()
        self.assertIsInstance(images[0], glance.notifier.ImageProxy)
        self.assertEqual('images_from_list', images[0].repo)

    def test_image_get_data_should_call_next_image_get_data(self):
        with mock.patch.object(self.image, 'get_data') as get_data_mock:
            self.image_proxy.get_data()

            self.assertTrue(get_data_mock.called)

    def test_image_get_data_notification(self):
        self.image_proxy.size = 10
        data = ''.join(self.image_proxy.get_data())
        self.assertEqual('0123456789', data)
        output_logs = self.notifier.get_logs()
        self.assertEqual(1, len(output_logs))
        output_log = output_logs[0]
        self.assertEqual('INFO', output_log['notification_type'])
        self.assertEqual('image.send', output_log['event_type'])
        self.assertEqual(self.image.image_id,
                         output_log['payload']['image_id'])
        self.assertEqual(TENANT2, output_log['payload']['receiver_tenant_id'])
        self.assertEqual(USER1, output_log['payload']['receiver_user_id'])
        self.assertEqual(10, output_log['payload']['bytes_sent'])
        self.assertEqual(TENANT1, output_log['payload']['owner_id'])

    def test_image_get_data_notification_disabled(self):
        self.config(disabled_notifications=['image.send'])
        self.image_proxy.size = 10
        data = ''.join(self.image_proxy.get_data())
        self.assertEqual('0123456789', data)
        output_logs = self.notifier.get_logs()
        self.assertEqual(0, len(output_logs))

    def test_image_get_data_size_mismatch(self):
        self.image_proxy.size = 11
        list(self.image_proxy.get_data())
        output_logs = self.notifier.get_logs()
        self.assertEqual(1, len(output_logs))
        output_log = output_logs[0]
        self.assertEqual('ERROR', output_log['notification_type'])
        self.assertEqual('image.send', output_log['event_type'])
        self.assertEqual(self.image.image_id,
                         output_log['payload']['image_id'])

    def test_image_set_data_prepare_notification(self):
        insurance = {'called': False}

        def data_iterator():
            output_logs = self.notifier.get_logs()
            self.assertEqual(1, len(output_logs))
            output_log = output_logs[0]
            self.assertEqual('INFO', output_log['notification_type'])
            self.assertEqual('image.prepare', output_log['event_type'])
            self.assertEqual(self.image.image_id, output_log['payload']['id'])
            self.assertEqual(['store1', 'store2'], output_log['payload'][
                'os_glance_importing_to_stores'])
            self.assertEqual([],
                             output_log['payload']['os_glance_failed_import'])
            yield 'abcd'
            yield 'efgh'
            insurance['called'] = True

        self.image_proxy.extra_properties[
            'os_glance_importing_to_stores'] = 'store1,store2'
        self.image_proxy.extra_properties['os_glance_failed_import'] = ''
        self.image_proxy.set_data(data_iterator(), 8)
        self.assertTrue(insurance['called'])

    def test_image_set_data_prepare_notification_disabled(self):
        insurance = {'called': False}

        def data_iterator():
            output_logs = self.notifier.get_logs()
            self.assertEqual(0, len(output_logs))
            yield 'abcd'
            yield 'efgh'
            insurance['called'] = True

        self.config(disabled_notifications=['image.prepare'])
        self.image_proxy.set_data(data_iterator(), 8)
        self.assertTrue(insurance['called'])

    def test_image_set_data_upload_and_activate_notification(self):
        image = ImageStub(image_id=UUID1, name='image-1', status='queued',
                          created_at=DATETIME, updated_at=DATETIME,
                          owner=TENANT1, visibility='public')
        context = glance.context.RequestContext(tenant=TENANT2, user=USER1)
        fake_notifier = unit_test_utils.FakeNotifier()
        image_proxy = glance.notifier.ImageProxy(image, context, fake_notifier)

        def data_iterator():
            fake_notifier.log = []
            yield 'abcde'
            yield 'fghij'
            image_proxy.extra_properties[
                'os_glance_importing_to_stores'] = 'store2'

        image_proxy.extra_properties[
            'os_glance_importing_to_stores'] = 'store1,store2'
        image_proxy.extra_properties['os_glance_failed_import'] = ''
        image_proxy.set_data(data_iterator(), 10)

        output_logs = fake_notifier.get_logs()
        self.assertEqual(2, len(output_logs))

        output_log = output_logs[0]
        self.assertEqual('INFO', output_log['notification_type'])
        self.assertEqual('image.upload', output_log['event_type'])
        self.assertEqual(self.image.image_id, output_log['payload']['id'])
        self.assertEqual(['store2'], output_log['payload'][
            'os_glance_importing_to_stores'])
        self.assertEqual([],
                         output_log['payload']['os_glance_failed_import'])

        output_log = output_logs[1]
        self.assertEqual('INFO', output_log['notification_type'])
        self.assertEqual('image.activate', output_log['event_type'])
        self.assertEqual(self.image.image_id, output_log['payload']['id'])

    def test_image_set_data_upload_and_not_activate_notification(self):
        insurance = {'called': False}

        def data_iterator():
            self.notifier.log = []
            yield 'abcde'
            yield 'fghij'
            self.image_proxy.extra_properties[
                'os_glance_importing_to_stores'] = 'store2'
            insurance['called'] = True

        self.image_proxy.set_data(data_iterator(), 10)

        output_logs = self.notifier.get_logs()
        self.assertEqual(1, len(output_logs))

        output_log = output_logs[0]
        self.assertEqual('INFO', output_log['notification_type'])
        self.assertEqual('image.upload', output_log['event_type'])
        self.assertEqual(self.image.image_id, output_log['payload']['id'])
        self.assertTrue(insurance['called'])

    def test_image_set_data_upload_and_activate_notification_disabled(self):
        insurance = {'called': False}
        image = ImageStub(image_id=UUID1, name='image-1', status='queued',
                          created_at=DATETIME, updated_at=DATETIME,
                          owner=TENANT1, visibility='public')
        context = glance.context.RequestContext(tenant=TENANT2, user=USER1)
        fake_notifier = unit_test_utils.FakeNotifier()
        image_proxy = glance.notifier.ImageProxy(image, context, fake_notifier)

        def data_iterator():
            fake_notifier.log = []
            yield 'abcde'
            yield 'fghij'
            insurance['called'] = True

        self.config(disabled_notifications=['image.activate', 'image.upload'])
        image_proxy.set_data(data_iterator(), 10)
        self.assertTrue(insurance['called'])
        output_logs = fake_notifier.get_logs()
        self.assertEqual(0, len(output_logs))

    def test_image_set_data_storage_full(self):
        def data_iterator():
            self.notifier.log = []
            yield 'abcde'
            raise glance_store.StorageFull(message='Modern Major General')

        self.assertRaises(webob.exc.HTTPRequestEntityTooLarge,
                          self.image_proxy.set_data, data_iterator(), 10)
        output_logs = self.notifier.get_logs()
        self.assertEqual(1, len(output_logs))

        output_log = output_logs[0]
        self.assertEqual('ERROR', output_log['notification_type'])
        self.assertEqual('image.upload', output_log['event_type'])
        self.assertIn('Modern Major General', output_log['payload'])

    def test_image_set_data_value_error(self):
        def data_iterator():
            self.notifier.log = []
            yield 'abcde'
            raise ValueError('value wrong')

        self.assertRaises(webob.exc.HTTPBadRequest,
                          self.image_proxy.set_data, data_iterator(), 10)

        output_logs = self.notifier.get_logs()
        self.assertEqual(1, len(output_logs))

        output_log = output_logs[0]
        self.assertEqual('ERROR', output_log['notification_type'])
        self.assertEqual('image.upload', output_log['event_type'])
        self.assertIn('value wrong', output_log['payload'])

    def test_image_set_data_duplicate(self):
        def data_iterator():
            self.notifier.log = []
            yield 'abcde'
            raise exception.Duplicate('Cant have duplicates')

        self.assertRaises(webob.exc.HTTPConflict,
                          self.image_proxy.set_data, data_iterator(), 10)

        output_logs = self.notifier.get_logs()
        self.assertEqual(1, len(output_logs))

        output_log = output_logs[0]
        self.assertEqual('ERROR', output_log['notification_type'])
        self.assertEqual('image.upload', output_log['event_type'])
        self.assertIn('Cant have duplicates', output_log['payload'])

    def test_image_set_data_storage_write_denied(self):
        def data_iterator():
            self.notifier.log = []
            yield 'abcde'
            raise glance_store.StorageWriteDenied(message='The Very Model')

        self.assertRaises(webob.exc.HTTPServiceUnavailable,
                          self.image_proxy.set_data, data_iterator(), 10)

        output_logs = self.notifier.get_logs()
        self.assertEqual(1, len(output_logs))

        output_log = output_logs[0]
        self.assertEqual('ERROR', output_log['notification_type'])
        self.assertEqual('image.upload', output_log['event_type'])
        self.assertIn('The Very Model', output_log['payload'])

    def test_image_set_data_forbidden(self):
        def data_iterator():
            self.notifier.log = []
            yield 'abcde'
            raise exception.Forbidden('Not allowed')

        self.assertRaises(webob.exc.HTTPForbidden,
                          self.image_proxy.set_data, data_iterator(), 10)

        output_logs = self.notifier.get_logs()
        self.assertEqual(1, len(output_logs))

        output_log = output_logs[0]
        self.assertEqual('ERROR', output_log['notification_type'])
        self.assertEqual('image.upload', output_log['event_type'])
        self.assertIn('Not allowed', output_log['payload'])

    def test_image_set_data_not_found(self):
        def data_iterator():
            self.notifier.log = []
            yield 'abcde'
            raise exception.NotFound('Not found')

        self.assertRaises(webob.exc.HTTPNotFound,
                          self.image_proxy.set_data, data_iterator(), 10)

        output_logs = self.notifier.get_logs()
        self.assertEqual(1, len(output_logs))

        output_log = output_logs[0]
        self.assertEqual('ERROR', output_log['notification_type'])
        self.assertEqual('image.upload', output_log['event_type'])
        self.assertIn('Not found', output_log['payload'])

    def test_image_set_data_HTTP_error(self):
        def data_iterator():
            self.notifier.log = []
            yield 'abcde'
            raise webob.exc.HTTPError('Http issue')

        self.assertRaises(webob.exc.HTTPError,
                          self.image_proxy.set_data, data_iterator(), 10)

        output_logs = self.notifier.get_logs()
        self.assertEqual(1, len(output_logs))

        output_log = output_logs[0]
        self.assertEqual('ERROR', output_log['notification_type'])
        self.assertEqual('image.upload', output_log['event_type'])
        self.assertIn('Http issue', output_log['payload'])

    def test_image_set_data_error(self):
        def data_iterator():
            self.notifier.log = []
            yield 'abcde'
            raise exception.GlanceException('Failed')

        self.assertRaises(exception.GlanceException,
                          self.image_proxy.set_data, data_iterator(), 10)

        output_logs = self.notifier.get_logs()
        self.assertEqual(1, len(output_logs))

        output_log = output_logs[0]
        self.assertEqual('ERROR', output_log['notification_type'])
        self.assertEqual('image.upload', output_log['event_type'])
        self.assertIn('Failed', output_log['payload'])


class TestImageMemberNotifications(utils.BaseTestCase):
    """Test Image Member Notifications work"""

    def setUp(self):
        super(TestImageMemberNotifications, self).setUp()
        self.context = glance.context.RequestContext(tenant=TENANT2,
                                                     user=USER1)
        self.notifier = unit_test_utils.FakeNotifier()

        self.image = ImageStub(
            image_id=UUID1, name='image-1', status='active', size=1024,
            created_at=DATETIME, updated_at=DATETIME, owner=TENANT1,
            visibility='public', container_format='ami',
            tags=['one', 'two'], disk_format='ami', min_ram=128,
            min_disk=10, checksum='ca425b88f047ce8ec45ee90e813ada91',
            locations=['http://127.0.0.1'])
        self.image_member = glance.domain.ImageMembership(
            id=1, image_id=UUID1, member_id=TENANT1, created_at=DATETIME,
            updated_at=DATETIME, status='accepted')

        self.image_member_repo_stub = ImageMemberRepoStub()
        self.image_member_repo_proxy = glance.notifier.ImageMemberRepoProxy(
            self.image_member_repo_stub, self.image,
            self.context, self.notifier)
        self.image_member_proxy = glance.notifier.ImageMemberProxy(
            self.image_member, self.context, self.notifier)

    def _assert_image_member_with_notifier(self, output_log, deleted=False):
        self.assertEqual(self.image_member.member_id,
                         output_log['payload']['member_id'])
        self.assertEqual(self.image_member.image_id,
                         output_log['payload']['image_id'])
        self.assertEqual(self.image_member.status,
                         output_log['payload']['status'])
        self.assertEqual(timeutils.isotime(self.image_member.created_at),
                         output_log['payload']['created_at'])
        self.assertEqual(timeutils.isotime(self.image_member.updated_at),
                         output_log['payload']['updated_at'])

        if deleted:
            self.assertTrue(output_log['payload']['deleted'])
            self.assertIsNotNone(output_log['payload']['deleted_at'])
        else:
            self.assertFalse(output_log['payload']['deleted'])
            self.assertIsNone(output_log['payload']['deleted_at'])

    def test_image_member_add_notification(self):
        self.image_member_repo_proxy.add(self.image_member_proxy)
        output_logs = self.notifier.get_logs()
        self.assertEqual(1, len(output_logs))
        output_log = output_logs[0]
        self.assertEqual('INFO', output_log['notification_type'])
        self.assertEqual('image.member.create', output_log['event_type'])
        self._assert_image_member_with_notifier(output_log)

    def test_image_member_add_notification_disabled(self):
        self.config(disabled_notifications=['image.member.create'])
        self.image_member_repo_proxy.add(self.image_member_proxy)
        output_logs = self.notifier.get_logs()
        self.assertEqual(0, len(output_logs))

    def test_image_member_save_notification(self):
        self.image_member_repo_proxy.save(self.image_member_proxy)
        output_logs = self.notifier.get_logs()
        self.assertEqual(1, len(output_logs))
        output_log = output_logs[0]
        self.assertEqual('INFO', output_log['notification_type'])
        self.assertEqual('image.member.update', output_log['event_type'])
        self._assert_image_member_with_notifier(output_log)

    def test_image_member_save_notification_disabled(self):
        self.config(disabled_notifications=['image.member.update'])
        self.image_member_repo_proxy.save(self.image_member_proxy)
        output_logs = self.notifier.get_logs()
        self.assertEqual(0, len(output_logs))

    def test_image_member_delete_notification(self):
        self.image_member_repo_proxy.remove(self.image_member_proxy)
        output_logs = self.notifier.get_logs()
        self.assertEqual(1, len(output_logs))
        output_log = output_logs[0]
        self.assertEqual('INFO', output_log['notification_type'])
        self.assertEqual('image.member.delete', output_log['event_type'])
        self._assert_image_member_with_notifier(output_log, deleted=True)

    def test_image_member_delete_notification_disabled(self):
        self.config(disabled_notifications=['image.member.delete'])
        self.image_member_repo_proxy.remove(self.image_member_proxy)
        output_logs = self.notifier.get_logs()
        self.assertEqual(0, len(output_logs))

    def test_image_member_get(self):
        image_member = self.image_member_repo_proxy.get(TENANT1)
        self.assertIsInstance(image_member, glance.notifier.ImageMemberProxy)
        self.assertEqual('image_member_from_get', image_member.repo)

    def test_image_member_list(self):
        image_members = self.image_member_repo_proxy.list()
        self.assertIsInstance(image_members[0],
                              glance.notifier.ImageMemberProxy)
        self.assertEqual('image_members_from_list', image_members[0].repo)


class TestTaskNotifications(utils.BaseTestCase):
    """Test Task Notifications work"""

    def setUp(self):
        super(TestTaskNotifications, self).setUp()
        task_input = {"loc": "fake"}
        self.task_stub = TaskStub(
            task_id='aaa',
            task_type='import',
            status='pending',
            owner=TENANT2,
            expires_at=None,
            created_at=DATETIME,
            updated_at=DATETIME,
            image_id='fake_image_id',
            user_id='fake_user',
            request_id='fake_request_id',
        )

        self.task = Task(
            task_id='aaa',
            task_type='import',
            status='pending',
            owner=TENANT2,
            expires_at=None,
            created_at=DATETIME,
            updated_at=DATETIME,
            task_input=task_input,
            result='res',
            message='blah',
            image_id='fake_image_id',
            user_id='fake_user',
            request_id='fake_request_id',
        )
        self.context = glance.context.RequestContext(
            tenant=TENANT2,
            user=USER1
        )
        self.task_repo_stub = TaskRepoStub()
        self.notifier = unit_test_utils.FakeNotifier()
        self.task_repo_proxy = glance.notifier.TaskRepoProxy(
            self.task_repo_stub,
            self.context,
            self.notifier
        )
        self.task_proxy = glance.notifier.TaskProxy(
            self.task,
            self.context,
            self.notifier
        )
        self.task_stub_proxy = glance.notifier.TaskStubProxy(
            self.task_stub,
            self.context,
            self.notifier
        )
        self.patcher = mock.patch('oslo_utils.timeutils.utcnow')
        mock_utcnow = self.patcher.start()
        mock_utcnow.return_value = datetime.datetime.now(
            datetime.timezone.utc).replace(tzinfo=None)

    def tearDown(self):
        super(TestTaskNotifications, self).tearDown()
        self.patcher.stop()

    def test_task_create_notification(self):
        self.task_repo_proxy.add(self.task_stub_proxy)
        output_logs = self.notifier.get_logs()
        self.assertEqual(1, len(output_logs))
        output_log = output_logs[0]
        self.assertEqual('INFO', output_log['notification_type'])
        self.assertEqual('task.create', output_log['event_type'])
        self.assertEqual(self.task.task_id, output_log['payload']['id'])
        self.assertEqual(
            timeutils.isotime(self.task.updated_at),
            output_log['payload']['updated_at']
        )
        self.assertEqual(
            timeutils.isotime(self.task.created_at),
            output_log['payload']['created_at']
        )
        if 'location' in output_log['payload']:
            self.fail('Notification contained location field.')
        # Verify newly added fields 'image_id', 'user_id' and
        # 'request_id' are not part of notification yet
        self.assertNotIn('image_id', output_log['payload'])
        self.assertNotIn('user_id', output_log['payload'])
        self.assertNotIn('request_id', output_log['payload'])

    def test_task_create_notification_disabled(self):
        self.config(disabled_notifications=['task.create'])
        self.task_repo_proxy.add(self.task_stub_proxy)
        output_logs = self.notifier.get_logs()
        self.assertEqual(0, len(output_logs))

    def test_task_delete_notification(self):
        now = timeutils.isotime()
        self.task_repo_proxy.remove(self.task_stub_proxy)
        output_logs = self.notifier.get_logs()
        self.assertEqual(1, len(output_logs))
        output_log = output_logs[0]
        self.assertEqual('INFO', output_log['notification_type'])
        self.assertEqual('task.delete', output_log['event_type'])
        self.assertEqual(self.task.task_id, output_log['payload']['id'])
        self.assertEqual(
            timeutils.isotime(self.task.updated_at),
            output_log['payload']['updated_at']
        )
        self.assertEqual(
            timeutils.isotime(self.task.created_at),
            output_log['payload']['created_at']
        )
        self.assertEqual(
            now,
            output_log['payload']['deleted_at']
        )
        if 'location' in output_log['payload']:
            self.fail('Notification contained location field.')
        # Verify newly added fields 'image_id', 'user_id' and
        # 'request_id' are not part of notification yet
        self.assertNotIn('image_id', output_log['payload'])
        self.assertNotIn('user_id', output_log['payload'])
        self.assertNotIn('request_id', output_log['payload'])

    def test_task_delete_notification_disabled(self):
        self.config(disabled_notifications=['task.delete'])
        self.task_repo_proxy.remove(self.task_stub_proxy)
        output_logs = self.notifier.get_logs()
        self.assertEqual(0, len(output_logs))

    def test_task_run_notification(self):
        with mock.patch('glance.async_.TaskExecutor') as mock_executor:
            executor = mock_executor.return_value
            executor._run.return_value = mock.Mock()
            self.task_proxy.run(executor=mock_executor)
        output_logs = self.notifier.get_logs()
        self.assertEqual(1, len(output_logs))
        output_log = output_logs[0]
        self.assertEqual('INFO', output_log['notification_type'])
        self.assertEqual('task.run', output_log['event_type'])
        self.assertEqual(self.task.task_id, output_log['payload']['id'])
        self.assertNotIn(self.task.image_id, output_log['payload'])
        self.assertNotIn(self.task.user_id, output_log['payload'])
        self.assertNotIn(self.task.request_id, output_log['payload'])

    def test_task_run_notification_disabled(self):
        self.config(disabled_notifications=['task.run'])
        with mock.patch('glance.async_.TaskExecutor') as mock_executor:
            executor = mock_executor.return_value
            executor._run.return_value = mock.Mock()
            self.task_proxy.run(executor=mock_executor)
        output_logs = self.notifier.get_logs()
        self.assertEqual(0, len(output_logs))

    def test_task_processing_notification(self):
        self.task_proxy.begin_processing()
        output_logs = self.notifier.get_logs()
        self.assertEqual(1, len(output_logs))
        output_log = output_logs[0]
        self.assertEqual('INFO', output_log['notification_type'])
        self.assertEqual('task.processing', output_log['event_type'])
        self.assertEqual(self.task.task_id, output_log['payload']['id'])
        # Verify newly added fields 'image_id', 'user_id' and
        # 'request_id' are not part of notification yet
        self.assertNotIn('image_id', output_log['payload'])
        self.assertNotIn('user_id', output_log['payload'])
        self.assertNotIn('request_id', output_log['payload'])

    def test_task_processing_notification_disabled(self):
        self.config(disabled_notifications=['task.processing'])
        self.task_proxy.begin_processing()
        output_logs = self.notifier.get_logs()
        self.assertEqual(0, len(output_logs))

    def test_task_success_notification(self):
        self.task_proxy.begin_processing()
        self.task_proxy.succeed(result=None)
        output_logs = self.notifier.get_logs()
        self.assertEqual(2, len(output_logs))
        output_log = output_logs[1]
        self.assertEqual('INFO', output_log['notification_type'])
        self.assertEqual('task.success', output_log['event_type'])
        self.assertEqual(self.task.task_id, output_log['payload']['id'])
        # Verify newly added fields 'image_id', 'user_id' and
        # 'request_id' are not part of notification yet
        self.assertNotIn('image_id', output_log['payload'])
        self.assertNotIn('user_id', output_log['payload'])
        self.assertNotIn('request_id', output_log['payload'])

    def test_task_success_notification_disabled(self):
        self.config(disabled_notifications=['task.processing', 'task.success'])
        self.task_proxy.begin_processing()
        self.task_proxy.succeed(result=None)
        output_logs = self.notifier.get_logs()
        self.assertEqual(0, len(output_logs))

    def test_task_failure_notification(self):
        self.task_proxy.fail(message=None)
        output_logs = self.notifier.get_logs()
        self.assertEqual(1, len(output_logs))
        output_log = output_logs[0]
        self.assertEqual('INFO', output_log['notification_type'])
        self.assertEqual('task.failure', output_log['event_type'])
        self.assertEqual(self.task.task_id, output_log['payload']['id'])
        # Verify newly added fields 'image_id', 'user_id' and
        # 'request_id' are not part of notification yet
        self.assertNotIn('image_id', output_log['payload'])
        self.assertNotIn('user_id', output_log['payload'])
        self.assertNotIn('request_id', output_log['payload'])

    def test_task_failure_notification_disabled(self):
        self.config(disabled_notifications=['task.failure'])
        self.task_proxy.fail(message=None)
        output_logs = self.notifier.get_logs()
        self.assertEqual(0, len(output_logs))



# Copyright 2012 OpenStack Foundation
# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import glance_store

from glance.api import policy
from glance.api import property_protections
from glance.common import property_utils
from glance.common import store_utils
import glance.db
import glance.domain
import glance.location
import glance.notifier
import glance.quota


class Gateway(object):
    def __init__(self, db_api=None, store_api=None, notifier=None,
                 policy_enforcer=None):
        self.db_api = db_api or glance.db.get_api()
        self.store_api = store_api or glance_store
        self.store_utils = store_utils
        self.notifier = notifier or glance.notifier.Notifier()
        self.policy = policy_enforcer or policy.Enforcer()
        self._property_rules = None

    def _get_property_rules(self):
        """Get cached PropertyRules instance, creating it if necessary.

        This method ensures that property protection rules are loaded
        only once per Gateway instance, rather than on every API request.
        """
        if self._property_rules is None:
            if property_utils.is_property_protection_enabled():
                self._property_rules = property_utils.PropertyRules(
                    self.policy)
        return self._property_rules

    def get_image_factory(self, context):
        factory = glance.domain.ImageFactory()
        factory = glance.location.ImageFactoryProxy(
            factory, context, self.store_api, self.store_utils)
        factory = glance.quota.ImageFactoryProxy(
            factory, context, self.db_api, self.store_utils)
        factory = glance.notifier.ImageFactoryProxy(
            factory, context, self.notifier)
        property_rules = self._get_property_rules()
        if property_rules is not None:
            factory = property_protections.ProtectedImageFactoryProxy(
                factory, context, property_rules)
        return factory

    def get_image_member_factory(self, context):
        factory = glance.domain.ImageMemberFactory()
        factory = glance.quota.ImageMemberFactoryProxy(
            factory, context, self.db_api, self.store_utils)
        return factory

    def get_repo(self, context):
        """Get the layered ImageRepo model.

        This is where we construct the "the onion" by layering
        ImageRepo models on top of each other, starting with the DB at
        the bottom.

        :param context: The RequestContext
        :returns: An ImageRepo-like object

        """
        repo = glance.db.ImageRepo(context, self.db_api)
        repo = glance.location.ImageRepoProxy(
            repo, context, self.store_api, self.store_utils)
        repo = glance.quota.ImageRepoProxy(
            repo, context, self.db_api, self.store_utils)
        repo = glance.notifier.ImageRepoProxy(
            repo, context, self.notifier)
        property_rules = self._get_property_rules()
        if property_rules is not None:
            repo = property_protections.ProtectedImageRepoProxy(
                repo, context, property_rules)

        return repo

    def get_member_repo(self, image, context):
        repo = glance.db.ImageMemberRepo(
            context, self.db_api, image)
        repo = glance.location.ImageMemberRepoProxy(
            repo, image, context, self.store_api)
        repo = glance.notifier.ImageMemberRepoProxy(
            repo, image, context, self.notifier)

        return repo

    def get_task_factory(self, context):
        factory = glance.domain.TaskFactory()
        factory = glance.notifier.TaskFactoryProxy(
            factory, context, self.notifier)
        return factory

    def get_task_repo(self, context):
        repo = glance.db.TaskRepo(context, self.db_api)
        repo = glance.notifier.TaskRepoProxy(
            repo, context, self.notifier)
        return repo

    def get_task_stub_repo(self, context):
        repo = glance.db.TaskRepo(context, self.db_api)
        repo = glance.notifier.TaskStubRepoProxy(
            repo, context, self.notifier)
        return repo

    def get_task_executor_factory(self, context, admin_context=None):
        task_repo = self.get_task_repo(context)
        image_repo = self.get_repo(context)
        image_factory = self.get_image_factory(context)
        if admin_context:
            admin_repo = self.get_repo(admin_context)
        else:
            admin_repo = None
        return glance.domain.TaskExecutorFactory(task_repo,
                                                 image_repo,
                                                 image_factory,
                                                 admin_repo=admin_repo)

    def get_metadef_namespace_factory(self, context):
        factory = glance.domain.MetadefNamespaceFactory()
        factory = glance.notifier.MetadefNamespaceFactoryProxy(
            factory, context, self.notifier)
        return factory

    def get_metadef_namespace_repo(self, context):
        """Get the layered NamespaceRepo model.

        This is where we construct the "the onion" by layering
        NamespaceRepo models on top of each other, starting with the DB at
        the bottom.

        :param context: The RequestContext
        :returns: An NamespaceRepo-like object
        """
        repo = glance.db.MetadefNamespaceRepo(context, self.db_api)
        repo = glance.notifier.MetadefNamespaceRepoProxy(
            repo, context, self.notifier)
        return repo

    def get_metadef_object_factory(self, context):
        factory = glance.domain.MetadefObjectFactory()
        factory = glance.notifier.MetadefObjectFactoryProxy(
            factory, context, self.notifier)
        return factory

    def get_metadef_object_repo(self, context):
        """Get the layered MetadefObjectRepo model.

        This is where we construct the "the onion" by layering
        MetadefObjectRepo models on top of each other, starting with the DB at
        the bottom.

        :param context: The RequestContext
        :returns: An MetadefObjectRepo-like object
        """
        repo = glance.db.MetadefObjectRepo(context, self.db_api)
        repo = glance.notifier.MetadefObjectRepoProxy(
            repo, context, self.notifier)
        return repo

    def get_metadef_resource_type_factory(self, context):
        factory = glance.domain.MetadefResourceTypeFactory()
        factory = glance.notifier.MetadefResourceTypeFactoryProxy(
            factory, context, self.notifier)
        return factory

    def get_metadef_resource_type_repo(self, context):
        """Get the layered MetadefResourceTypeRepo model.

        This is where we construct the "the onion" by layering
        MetadefResourceTypeRepo models on top of each other, starting with
        the DB at the bottom.

        :param context: The RequestContext
        :returns: An MetadefResourceTypeRepo-like object
        """
        repo = glance.db.MetadefResourceTypeRepo(
            context, self.db_api)
        repo = glance.notifier.MetadefResourceTypeRepoProxy(
            repo, context, self.notifier)
        return repo

    def get_metadef_property_factory(self, context):
        factory = glance.domain.MetadefPropertyFactory()
        factory = glance.notifier.MetadefPropertyFactoryProxy(
            factory, context, self.notifier)
        return factory

    def get_metadef_property_repo(self, context):
        """Get the layered MetadefPropertyRepo model.

        This is where we construct the "the onion" by layering
        MetadefPropertyRepo models on top of each other, starting with
        the DB at the bottom.

        :param context: The RequestContext
        :returns: An MetadefPropertyRepo-like object
        """
        repo = glance.db.MetadefPropertyRepo(context, self.db_api)
        repo = glance.notifier.MetadefPropertyRepoProxy(
            repo, context, self.notifier)
        return repo

    def get_metadef_tag_factory(self, context):
        factory = glance.domain.MetadefTagFactory()
        factory = glance.notifier.MetadefTagFactoryProxy(
            factory, context, self.notifier)
        return factory

    def get_metadef_tag_repo(self, context):
        """Get the layered MetadefTagRepo model.

        This is where we construct the "the onion" by layering
        MetadefTagRepo models on top of each other, starting with
        the DB at the bottom.

        :param context: The RequestContext
        :returns: An MetadefTagRepo-like object
        """
        repo = glance.db.MetadefTagRepo(context, self.db_api)
        repo = glance.notifier.MetadefTagRepoProxy(
            repo, context, self.notifier)
        return repo



# Copyright 2020 Red Hat, Inc
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from glance.i18n import _
from oslo_config import cfg

removed_opts = [
    cfg.BoolOpt('owner_is_tenant',
                default=True,
                help=_("""
This option has been removed in Wallaby.  Because there is no migration path
for installations that had owner_is_tenant==False, we have defined this option
so that the code can probe the config file and refuse to start the api service
if the deployment has been using that setting.
""")),
]


def register_removed_options():
    # NOTE(cyril): This should only be called when we need to use options that
    # have been removed and are therefore no longer relevant. This is the case
    # of upgrade checks, for instance.
    cfg.CONF.register_opts(removed_opts)



# Copyright 2020 Red Hat, Inc
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import glance_store
from oslo_config import cfg
from oslo_upgradecheck import upgradecheck

from glance.cmd.status import Checks
from glance.tests import utils as test_utils

CONF = cfg.CONF


class TestUpgradeChecks(test_utils.BaseTestCase):
    def setUp(self):
        super(TestUpgradeChecks, self).setUp()
        glance_store.register_opts(CONF)
        self.checker = Checks()

    def test_sheepdog_removal_no_config(self):
        self.assertEqual(self.checker._check_sheepdog_store().code,
                         upgradecheck.Code.SUCCESS)

    def test_sheepdog_removal_enabled_backends(self):
        self.config(enabled_backends=None)
        self.assertEqual(self.checker._check_sheepdog_store().code,
                         upgradecheck.Code.SUCCESS)

        self.config(enabled_backends={})
        self.assertEqual(self.checker._check_sheepdog_store().code,
                         upgradecheck.Code.SUCCESS)

        self.config(enabled_backends={'foo': 'bar'})
        self.assertEqual(self.checker._check_sheepdog_store().code,
                         upgradecheck.Code.SUCCESS)

        self.config(enabled_backends={'sheepdog': 'foobar'})
        self.assertEqual(self.checker._check_sheepdog_store().code,
                         upgradecheck.Code.FAILURE)

    def test_sheepdog_removal_glance_store_stores(self):
        self.config(stores=None, group='glance_store')
        self.assertEqual(self.checker._check_sheepdog_store().code,
                         upgradecheck.Code.SUCCESS)

        self.config(stores='', group='glance_store')
        self.assertEqual(self.checker._check_sheepdog_store().code,
                         upgradecheck.Code.SUCCESS)

        self.config(stores='foo', group='glance_store')
        self.assertEqual(self.checker._check_sheepdog_store().code,
                         upgradecheck.Code.SUCCESS)

        self.config(stores='sheepdog', group='glance_store')
        self.assertEqual(self.checker._check_sheepdog_store().code,
                         upgradecheck.Code.FAILURE)

    def test_owner_is_tenant_removal(self):
        self.config(owner_is_tenant=True)
        self.assertEqual(self.checker._check_owner_is_tenant().code,
                         upgradecheck.Code.SUCCESS)

        self.config(owner_is_tenant=False)
        self.assertEqual(self.checker._check_owner_is_tenant().code,
                         upgradecheck.Code.FAILURE)



# Copyright 2020 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from glance.api import property_protections
from glance.common import property_utils
from glance import context
from glance import gateway
from glance import notifier
from glance import quota
from glance.tests.unit import utils as unit_test_utils
import glance.tests.utils as test_utils


class TestGateway(test_utils.BaseTestCase):
    def setUp(self):
        super(TestGateway, self).setUp()
        self.gateway = gateway.Gateway()
        self.context = mock.sentinel.context

    @mock.patch('glance.domain.TaskExecutorFactory')
    def test_get_task_executor_factory(self, mock_factory):
        @mock.patch.object(self.gateway, 'get_task_repo')
        @mock.patch.object(self.gateway, 'get_repo')
        @mock.patch.object(self.gateway, 'get_image_factory')
        def _test(mock_gif, mock_gr, mock_gtr):
            self.gateway.get_task_executor_factory(self.context)
            mock_gtr.assert_called_once_with(self.context)
            mock_gr.assert_called_once_with(self.context)
            mock_gif.assert_called_once_with(self.context)
            mock_factory.assert_called_once_with(
                mock_gtr.return_value,
                mock_gr.return_value,
                mock_gif.return_value,
                admin_repo=None)

        _test()

    @mock.patch('glance.domain.TaskExecutorFactory')
    def test_get_task_executor_factory_with_admin(self, mock_factory):
        @mock.patch.object(self.gateway, 'get_task_repo')
        @mock.patch.object(self.gateway, 'get_repo')
        @mock.patch.object(self.gateway, 'get_image_factory')
        def _test(mock_gif, mock_gr, mock_gtr):
            mock_gr.side_effect = [mock.sentinel.image_repo,
                                   mock.sentinel.admin_repo]
            self.gateway.get_task_executor_factory(
                self.context,
                admin_context=mock.sentinel.admin_context)
            mock_gtr.assert_called_once_with(self.context)
            mock_gr.assert_has_calls([
                mock.call(self.context),
                mock.call(mock.sentinel.admin_context)
            ])
            mock_gif.assert_called_once_with(self.context)
            mock_factory.assert_called_once_with(
                mock_gtr.return_value,
                mock.sentinel.image_repo,
                mock_gif.return_value,
                admin_repo=mock.sentinel.admin_repo)

        _test()

    def test_get_repo(self):
        repo = self.gateway.get_repo(self.context)
        self.assertIsInstance(repo, notifier.ImageRepoProxy)

    @mock.patch('glance.common.property_utils.PropertyRules._load_rules')
    def test_get_repo_with_pp(self, mock_load):
        self.config(property_protection_file='foo')
        repo = self.gateway.get_repo(self.context)
        self.assertIsInstance(repo,
                              property_protections.ProtectedImageRepoProxy)

    def test_get_image_factory(self):
        factory = self.gateway.get_image_factory(self.context)
        self.assertIsInstance(factory, notifier.ImageFactoryProxy)

    @mock.patch('glance.common.property_utils.PropertyRules._load_rules')
    def test_get_image_factory_with_pp(self, mock_load):
        self.config(property_protection_file='foo')
        factory = self.gateway.get_image_factory(self.context)
        self.assertIsInstance(factory,
                              property_protections.ProtectedImageFactoryProxy)

    def test_get_repo_member_property(self):
        """Test that the image.member property is propagated all the way from
        the DB to the top of the gateway repo stack.
        """
        db_api = unit_test_utils.FakeDB()
        gw = gateway.Gateway(db_api=db_api)

        # Get the UUID1 image as TENANT1
        ctxt = context.RequestContext(tenant=unit_test_utils.TENANT1)
        repo = gw.get_repo(ctxt)
        image = repo.get(unit_test_utils.UUID1)
        # We own the image, so member is None
        self.assertIsNone(image.member)

        # Get the UUID1 image as TENANT2
        ctxt = context.RequestContext(tenant=unit_test_utils.TENANT2)
        repo = gw.get_repo(ctxt)
        image = repo.get(unit_test_utils.UUID1)
        # We are a member, so member is our tenant id
        self.assertEqual(unit_test_utils.TENANT2, image.member)

    def test_get_namespace_repo(self):
        repo = self.gateway.get_metadef_namespace_repo(self.context)
        self.assertIsInstance(repo, notifier.MetadefNamespaceRepoProxy)

    def test_get_namespace_factory(self):
        repo = self.gateway.get_metadef_namespace_factory(self.context)
        self.assertIsInstance(repo, notifier.MetadefNamespaceFactoryProxy)

    def test_get_object_repo(self):
        repo = self.gateway.get_metadef_object_repo(self.context)
        self.assertIsInstance(repo, notifier.MetadefObjectRepoProxy)

    def test_get_object_factory(self):
        repo = self.gateway.get_metadef_object_factory(self.context)
        self.assertIsInstance(repo, notifier.MetadefObjectFactoryProxy)

    def test_get_resourcetype_repo(self):
        repo = self.gateway.get_metadef_resource_type_repo(self.context)
        self.assertIsInstance(repo, notifier.MetadefResourceTypeRepoProxy)

    def test_get_resource_type_factory(self):
        repo = self.gateway.get_metadef_resource_type_factory(self.context)
        self.assertIsInstance(repo, notifier.MetadefResourceTypeFactoryProxy)

    def test_get_property_repo(self):
        repo = self.gateway.get_metadef_property_repo(self.context)
        self.assertIsInstance(repo, notifier.MetadefPropertyRepoProxy)

    def test_get_property_factory(self):
        repo = self.gateway.get_metadef_property_factory(self.context)
        self.assertIsInstance(repo, notifier.MetadefPropertyFactoryProxy)

    def test_get_tag_repo(self):
        repo = self.gateway.get_metadef_tag_repo(self.context)
        self.assertIsInstance(repo, notifier.MetadefTagRepoProxy)

    def test_get_tag_factory(self):
        repo = self.gateway.get_metadef_tag_factory(self.context)
        self.assertIsInstance(repo, notifier.MetadefTagFactoryProxy)

    def test_get_member_repo(self):
        repo = self.gateway.get_member_repo(mock.sentinel.image, self.context)
        self.assertIsInstance(repo, notifier.ImageMemberRepoProxy)

    def test_get_member_factory(self):
        repo = self.gateway.get_image_member_factory(self.context)
        self.assertIsInstance(repo, quota.ImageMemberFactoryProxy)

    def test_get_task_repo(self):
        repo = self.gateway.get_task_repo(self.context)
        self.assertIsInstance(repo, notifier.TaskRepoProxy)

    def test_get_task_factory(self):
        repo = self.gateway.get_task_factory(self.context)
        self.assertIsInstance(repo, notifier.TaskFactoryProxy)

    def test_get_task_stub_repo(self):
        repo = self.gateway.get_task_stub_repo(self.context)
        self.assertIsInstance(repo, notifier.TaskStubRepoProxy)

    @mock.patch('glance.common.property_utils.PropertyRules._load_rules')
    def test_get_property_rules_caches_instance(self, mock_load):
        """Test that _get_property_rules caches PropertyRules instance."""
        self.config(property_protection_file='foo')

        # First call should create PropertyRules
        rules1 = self.gateway._get_property_rules()
        self.assertIsNotNone(rules1)
        self.assertIsInstance(rules1, property_utils.PropertyRules)
        self.assertEqual(1, mock_load.call_count)

        # Second call should return the same cached instance
        rules2 = self.gateway._get_property_rules()
        self.assertIs(rules1, rules2)
        # _load_rules should not be called again
        self.assertEqual(1, mock_load.call_count)

    def test_get_property_rules_returns_none_when_disabled(self):
        """Test that _get_property_rules returns None when PP is disabled."""
        self.config(property_protection_file=None)
        rules = self.gateway._get_property_rules()
        self.assertIsNone(rules)



# Copyright (c) 2017 Huawei Technologies Co., Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Tests for the sqlite image_cache driver.
"""

import os
from unittest import mock

import ddt

from glance.image_cache.drivers import sqlite
from glance.tests import utils


@ddt.ddt
class TestSqlite(utils.BaseTestCase):

    @ddt.data(True, False)
    def test_delete_cached_file(self, throw_not_exists):

        with mock.patch.object(os, 'unlink') as mock_unlink:
            if throw_not_exists:
                mock_unlink.side_effect = OSError((2, 'File not found'))

        # Should not raise an exception in all cases
        sqlite.delete_cached_file('/tmp/dummy_file')



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import glance.async_.flows.plugins.image_conversion
import glance.async_.flows.plugins.inject_image_metadata


# Note(jokke): This list contains tuples of config options for import plugins.
# When new plugin is introduced its config options need to be added to this
# list so that they can be processed, when config generator is used to generate
# the glance-image-import.conf.sample it will also pick up the details. The
# module needs to be imported as the Glance release packaged example(s) above
# and the first part of the tuple refers to the group the options gets
# registered under at the config file.
PLUGIN_OPTS = [
    ('inject_metadata_properties',
     glance.async_.flows.plugins.inject_image_metadata.inject_metadata_opts),
    ('image_conversion',
     glance.async_.flows.plugins.image_conversion.conversion_plugin_opts),
]


def get_plugin_opts():
    return PLUGIN_OPTS



# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import http.client as http
import urllib.parse as urlparse

import debtcollector
import glance_store
from oslo_config import cfg
from oslo_log import log as logging
import oslo_serialization.jsonutils as json
from oslo_utils import uuidutils
import webob.exc

from glance.api import common
from glance.api import policy
from glance.api.v2 import policy as api_policy
from glance.common import exception
from glance.common import timeutils
from glance.common import wsgi
import glance.db
import glance.gateway
from glance.i18n import _, _LW
import glance.notifier
import glance.schema

LOG = logging.getLogger(__name__)

CONF = cfg.CONF
CONF.import_opt('task_time_to_live', 'glance.common.config', group='task')

_DEPRECATION_MESSAGE = ("The task API is being deprecated and "
                        "it will be superseded by the new image import "
                        "API. Please refer to this link for more "
                        "information about the aforementioned process: "
                        "https://specs.openstack.org/openstack/glance-specs/"
                        "specs/mitaka/approved/image-import/"
                        "image-import-refactor.html")


class TasksController(object):
    """Manages operations on tasks."""

    def __init__(self, db_api=None, policy_enforcer=None, notifier=None,
                 store_api=None):
        self.db_api = db_api or glance.db.get_api()
        self.policy = policy_enforcer or policy.Enforcer()
        self.notifier = notifier or glance.notifier.Notifier()
        self.store_api = store_api or glance_store
        self.gateway = glance.gateway.Gateway(self.db_api, self.store_api,
                                              self.notifier, self.policy)

    @debtcollector.removals.remove(message=_DEPRECATION_MESSAGE)
    def create(self, req, task):
        # NOTE(rosmaita): access to this call is enforced in the deserializer

        ctxt = req.context
        task_factory = self.gateway.get_task_factory(ctxt)
        executor_factory = self.gateway.get_task_executor_factory(ctxt)
        task_repo = self.gateway.get_task_repo(ctxt)
        try:
            new_task = task_factory.new_task(
                task_type=task['type'],
                owner=ctxt.owner,
                task_input=task['input'],
                image_id=task['input'].get('image_id'),
                user_id=ctxt.user_id,
                request_id=ctxt.request_id)
            task_repo.add(new_task)
            task_executor = executor_factory.new_task_executor(ctxt)
            pool = common.get_thread_pool("tasks_pool")
            pool.spawn(new_task.run, task_executor)
        except exception.Forbidden as e:
            msg = (_LW("Forbidden to create task. Reason: %(reason)s")
                   % {'reason': e})
            LOG.warning(msg)
            raise webob.exc.HTTPForbidden(explanation=e.msg)
        return new_task

    @debtcollector.removals.remove(message=_DEPRECATION_MESSAGE)
    def index(self, req, marker=None, limit=None, sort_key='created_at',
              sort_dir='desc', filters=None):
        # NOTE(rosmaita): access to this call is enforced in the deserializer

        result = {}
        if filters is None:
            filters = {}
        filters['deleted'] = False

        if limit is None:
            limit = CONF.limit_param_default
        limit = min(CONF.api_limit_max, limit)

        task_repo = self.gateway.get_task_stub_repo(req.context)
        try:
            tasks = task_repo.list(marker, limit, sort_key,
                                   sort_dir, filters)
            if len(tasks) != 0 and len(tasks) == limit:
                result['next_marker'] = tasks[-1].task_id
        except (exception.NotFound, exception.InvalidSortKey,
                exception.InvalidFilterRangeValue) as e:
            LOG.warning(str(e))
            raise webob.exc.HTTPBadRequest(explanation=e.msg)
        except exception.Forbidden as e:
            LOG.warning(str(e))
            raise webob.exc.HTTPForbidden(explanation=e.msg)
        result['tasks'] = tasks
        return result

    @debtcollector.removals.remove(message=_DEPRECATION_MESSAGE)
    def get(self, req, task_id):
        _enforce_access_policy(self.policy, req)
        try:
            task_repo = self.gateway.get_task_repo(req.context)
            task = task_repo.get(task_id)
        except exception.NotFound as e:
            msg = (_LW("Failed to find task %(task_id)s. Reason: %(reason)s")
                   % {'task_id': task_id,
                      'reason': e})
            LOG.warning(msg)
            raise webob.exc.HTTPNotFound(explanation=e.msg)
        except exception.Forbidden as e:
            msg = (_LW("Forbidden to get task %(task_id)s. Reason:"
                       " %(reason)s")
                   % {'task_id': task_id,
                      'reason': e})
            LOG.warning(msg)
            raise webob.exc.HTTPForbidden(explanation=e.msg)
        return task

    @debtcollector.removals.remove(message=_DEPRECATION_MESSAGE)
    def delete(self, req, task_id):
        _enforce_access_policy(self.policy, req)
        msg = (_("This operation is currently not permitted on Glance Tasks. "
                 "They are auto deleted after reaching the time based on "
                 "their expires_at property."))
        raise webob.exc.HTTPMethodNotAllowed(explanation=msg,
                                             headers={'Allow': 'GET'},
                                             body_template='${explanation}')


class RequestDeserializer(wsgi.JSONRequestDeserializer):
    _required_properties = ['type', 'input']

    def _get_request_body(self, request):
        output = super(RequestDeserializer, self).default(request)
        if 'body' not in output:
            msg = _('Body expected in request.')
            raise webob.exc.HTTPBadRequest(explanation=msg)
        return output['body']

    def _validate_sort_dir(self, sort_dir):
        if sort_dir not in ['asc', 'desc']:
            msg = _('Invalid sort direction: %s') % sort_dir
            raise webob.exc.HTTPBadRequest(explanation=msg)

        return sort_dir

    def _get_filters(self, filters):
        status = filters.get('status')
        if status:
            if status not in ['pending', 'processing', 'success', 'failure']:
                msg = _('Invalid status value: %s') % status
                raise webob.exc.HTTPBadRequest(explanation=msg)

        type = filters.get('type')
        if type:
            if type not in ['import']:
                msg = _('Invalid type value: %s') % type
                raise webob.exc.HTTPBadRequest(explanation=msg)

        return filters

    def _validate_marker(self, marker):
        if marker and not uuidutils.is_uuid_like(marker):
            msg = _('Invalid marker format')
            raise webob.exc.HTTPBadRequest(explanation=msg)
        return marker

    def _validate_limit(self, limit):
        try:
            limit = int(limit)
        except ValueError:
            msg = _("limit param must be an integer")
            raise webob.exc.HTTPBadRequest(explanation=msg)

        if limit < 0:
            msg = _("limit param must be positive")
            raise webob.exc.HTTPBadRequest(explanation=msg)

        return limit

    def _validate_create_body(self, body):
        """Validate the body of task creating request"""
        for param in self._required_properties:
            if param not in body:
                msg = _("Task '%s' is required") % param
                raise webob.exc.HTTPBadRequest(explanation=msg)

    def __init__(self, schema=None, policy_engine=None):
        super(RequestDeserializer, self).__init__()
        self.schema = schema or get_task_schema()
        # want to enforce the access policy as early as possible
        self.policy_engine = policy_engine or policy.Enforcer()

    def create(self, request):
        _enforce_access_policy(self.policy_engine, request)
        body = self._get_request_body(request)
        self._validate_create_body(body)
        try:
            self.schema.validate(body)
        except exception.InvalidObject as e:
            raise webob.exc.HTTPBadRequest(explanation=e.msg)
        task = {}
        properties = body
        for key in self._required_properties:
            try:
                task[key] = properties.pop(key)
            except KeyError:
                pass
        return dict(task=task)

    def index(self, request):
        _enforce_access_policy(self.policy_engine, request)
        params = request.params.copy()
        limit = params.pop('limit', None)
        marker = params.pop('marker', None)
        sort_dir = params.pop('sort_dir', 'desc')
        query_params = {
            'sort_key': params.pop('sort_key', 'created_at'),
            'sort_dir': self._validate_sort_dir(sort_dir),
            'filters': self._get_filters(params)
        }

        if marker is not None:
            query_params['marker'] = self._validate_marker(marker)

        if limit is not None:
            query_params['limit'] = self._validate_limit(limit)
        return query_params


class ResponseSerializer(wsgi.JSONResponseSerializer):
    def __init__(self, task_schema=None, partial_task_schema=None):
        super(ResponseSerializer, self).__init__()
        self.task_schema = task_schema or get_task_schema()
        self.partial_task_schema = (partial_task_schema
                                    or _get_partial_task_schema())

    def _inject_location_header(self, response, task):
        location = self._get_task_location(task)
        response.headers['Location'] = location

    def _get_task_location(self, task):
        return '/v2/tasks/%s' % task.task_id

    def _format_task(self, schema, task):
        task_view = {
            'id': task.task_id,
            'input': task.task_input,
            'type': task.type,
            'status': task.status,
            'owner': task.owner,
            'message': task.message,
            'result': task.result,
            'created_at': timeutils.isotime(task.created_at),
            'updated_at': timeutils.isotime(task.updated_at),
            'self': self._get_task_location(task),
            'schema': '/v2/schemas/task'
        }
        if task.image_id:
            task_view['image_id'] = task.image_id
        if task.request_id:
            task_view['request_id'] = task.request_id
        if task.user_id:
            task_view['user_id'] = task.user_id
        if task.expires_at:
            task_view['expires_at'] = timeutils.isotime(task.expires_at)
        task_view = schema.filter(task_view)  # domain
        return task_view

    def _format_task_stub(self, schema, task):
        task_view = {
            'id': task.task_id,
            'type': task.type,
            'status': task.status,
            'owner': task.owner,
            'created_at': timeutils.isotime(task.created_at),
            'updated_at': timeutils.isotime(task.updated_at),
            'self': self._get_task_location(task),
            'schema': '/v2/schemas/task'
        }
        if task.expires_at:
            task_view['expires_at'] = timeutils.isotime(task.expires_at)
        task_view = schema.filter(task_view)  # domain
        return task_view

    def create(self, response, task):
        response.status_int = http.CREATED
        self._inject_location_header(response, task)
        self.get(response, task)

    def get(self, response, task):
        task_view = self._format_task(self.task_schema, task)
        response.unicode_body = json.dumps(task_view, ensure_ascii=False)
        response.content_type = 'application/json'

    def index(self, response, result):
        params = dict(response.request.params)
        params.pop('marker', None)
        query = urlparse.urlencode(params)
        body = {
            'tasks': [self._format_task_stub(self.partial_task_schema, task)
                      for task in result['tasks']],
            'first': '/v2/tasks',
            'schema': '/v2/schemas/tasks',
        }
        if query:
            body['first'] = '%s?%s' % (body['first'], query)
        if 'next_marker' in result:
            params['marker'] = result['next_marker']
            next_query = urlparse.urlencode(params)
            body['next'] = '/v2/tasks?%s' % next_query
        response.unicode_body = json.dumps(body, ensure_ascii=False)
        response.content_type = 'application/json'


_TASK_SCHEMA = {
    "id": {
        "description": _("An identifier for the task"),
        "pattern": _('^([0-9a-fA-F]){8}-([0-9a-fA-F]){4}-([0-9a-fA-F]){4}'
                     '-([0-9a-fA-F]){4}-([0-9a-fA-F]){12}$'),
        "type": "string"
    },
    "type": {
        "description": _("The type of task represented by this content"),
        "enum": [
            "import",
            "api_image_import",
            "location_import",
        ],
        "type": "string"
    },
    "status": {
        "description": _("The current status of this task"),
        "enum": [
            "pending",
            "processing",
            "success",
            "failure"
        ],
        "type": "string"
    },
    "input": {
        "description": _("The parameters required by task, JSON blob"),
        "type": ["null", "object"],
    },
    "result": {
        "description": _("The result of current task, JSON blob"),
        "type": ["null", "object"],
    },
    "owner": {
        "description": _("An identifier for the owner of this task"),
        "type": "string"
    },
    "message": {
        "description": _("Human-readable informative message only included"
                         " when appropriate (usually on failure)"),
        "type": "string",
    },
    "image_id": {
        "description": _("Image associated with the task"),
        "type": "string",
    },
    "request_id": {
        "description": _("Human-readable informative request-id"),
        "type": "string",
    },
    "user_id": {
        "description": _("User associated with the task"),
        "type": "string",
    },
    "expires_at": {
        "description": _("Datetime when this resource would be"
                         " subject to removal"),
        "type": ["null", "string"]
    },
    "created_at": {
        "description": _("Datetime when this resource was created"),
        "type": "string"
    },
    "updated_at": {
        "description": _("Datetime when this resource was updated"),
        "type": "string"
    },
    'self': {
        'readOnly': True,
        'type': 'string'
    },
    'schema': {
        'readOnly': True,
        'type': 'string'
    }
}


def _enforce_access_policy(policy_engine, request):
    api_policy.TasksAPIPolicy(
        request.context,
        enforcer=policy_engine).tasks_api_access()


def get_task_schema():
    properties = copy.deepcopy(_TASK_SCHEMA)
    schema = glance.schema.Schema('task', properties)
    return schema


def _get_partial_task_schema():
    properties = copy.deepcopy(_TASK_SCHEMA)
    hide_properties = ['input', 'result', 'message']
    for key in hide_properties:
        del properties[key]
    schema = glance.schema.Schema('task', properties)
    return schema


def get_collection_schema():
    task_schema = _get_partial_task_schema()
    return glance.schema.CollectionSchema('tasks', task_schema)


def create_resource():
    """Task resource factory method"""
    task_schema = get_task_schema()
    partial_task_schema = _get_partial_task_schema()
    deserializer = RequestDeserializer(task_schema)
    serializer = ResponseSerializer(task_schema, partial_task_schema)
    controller = TasksController()
    return wsgi.Resource(controller, deserializer, serializer)



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import http.client as http

from oslo_serialization import jsonutils

from glance.tests.functional.v2 import metadef_base


class TestMetadefTags(metadef_base.MetadefFunctionalTestBase):

    def setUp(self):
        super(TestMetadefTags, self).setUp()
        self.start_server(enable_cache=False)

    def test_metadata_tags_lifecycle(self):
        # Namespace should not exist
        path = '/v2/metadefs/namespaces/MyNamespace'
        response = self.api_get(path, headers=self._headers())
        self.assertEqual(http.NOT_FOUND, response.status_code)

        # Create a namespace
        path = '/v2/metadefs/namespaces'
        headers = self._headers({'content-type': 'application/json'})
        namespace_name = 'MyNamespace'
        data = {
            "namespace": namespace_name,
            "display_name": "My User Friendly Namespace",
            "description": "My description",
            "visibility": "public",
            "protected": False,
            "owner": "The Test Owner"
        }
        response = self.api_post(path, headers=headers, json=data)
        self.assertEqual(http.CREATED, response.status_code)

        # Metadata tag should not exist
        metadata_tag_name = "tag1"
        path = '/v2/metadefs/namespaces/%s/tags/%s' % (
            namespace_name, metadata_tag_name)
        response = self.api_get(path, headers=self._headers())
        self.assertEqual(http.NOT_FOUND, response.status_code)

        # Create the metadata tag
        headers = self._headers({'content-type': 'application/json'})
        response = self.api_post(path, headers=headers)
        self.assertEqual(http.CREATED, response.status_code)

        # Get the metadata tag created above
        response = self.api_get(path,
                                headers=self._headers())
        self.assertEqual(http.OK, response.status_code)
        metadata_tag = jsonutils.loads(response.text)
        self.assertEqual(metadata_tag_name, metadata_tag['name'])

        # Returned tag should match the created tag
        checked_keys = set([
            'name',
            'created_at',
            'updated_at'
        ])
        self.assertEqual(checked_keys, set(metadata_tag.keys()))
        expected_metadata_tag = {
            "name": metadata_tag_name
        }

        # Simple key values
        checked_values = set([
            'name'
        ])
        for key, value in expected_metadata_tag.items():
            if key in checked_values:
                self.assertEqual(metadata_tag[key], value, key)

        # Try to create a duplicate metadata tag
        headers = self._headers({'content-type': 'application/json'})
        response = self.api_post(path, headers=headers)
        self.assertEqual(http.CONFLICT, response.status_code)

        # The metadata_tag should be mutable
        path = '/v2/metadefs/namespaces/%s/tags/%s' % (
            namespace_name, metadata_tag_name)
        media_type = 'application/json'
        headers = self._headers({'content-type': media_type})
        metadata_tag_name = "tag1-UPDATED"
        data = {
            "name": metadata_tag_name
        }
        response = self.api_put(path, headers=headers, json=data)
        self.assertEqual(http.OK, response.status_code, response.text)

        # Returned metadata_tag should reflect the changes
        metadata_tag = jsonutils.loads(response.text)
        self.assertEqual('tag1-UPDATED', metadata_tag['name'])

        # Updates should persist across requests
        path = '/v2/metadefs/namespaces/%s/tags/%s' % (
            namespace_name, metadata_tag_name)
        response = self.api_get(path, headers=self._headers())
        self.assertEqual(http.OK, response.status_code)
        self.assertEqual('tag1-UPDATED', metadata_tag['name'])

        # Deletion of metadata_tag_name
        path = '/v2/metadefs/namespaces/%s/tags/%s' % (
            namespace_name, metadata_tag_name)
        response = self.api_delete(path, headers=self._headers())
        self.assertEqual(http.NO_CONTENT, response.status_code)

        # metadata_tag_name should not exist
        path = '/v2/metadefs/namespaces/%s/tags/%s' % (
            namespace_name, metadata_tag_name)
        response = self.api_get(path, headers=self._headers())
        self.assertEqual(http.NOT_FOUND, response.status_code)

        # Create multiple tags.
        path = '/v2/metadefs/namespaces/%s/tags' % namespace_name
        headers = self._headers({'content-type': 'application/json'})
        data = {
            "tags": [
                {"name": "tag1"},
                {"name": "tag2"},
                {"name": "tag3"}
            ]
        }
        response = self.api_post(path, headers=headers, json=data)
        self.assertEqual(http.CREATED, response.status_code)

        # List out the three new tags.
        response = self.api_get(path, headers=self._headers())
        self.assertEqual(http.OK, response.status_code)
        tags = jsonutils.loads(response.text)['tags']
        self.assertEqual(3, len(tags))

        # Attempt to create bogus duplicate tag4
        data = {
            "tags": [
                {"name": "tag4"},
                {"name": "tag5"},
                {"name": "tag4"}
            ]
        }
        response = self.api_post(path, headers=headers, json=data)
        self.assertEqual(http.CONFLICT, response.status_code)

        # Verify the previous 3 still exist
        response = self.api_get(path, headers=self._headers())
        self.assertEqual(http.OK, response.status_code)
        tags = jsonutils.loads(response.text)['tags']
        self.assertEqual(3, len(tags))

        # Create new tags and append to existing tags.
        path = '/v2/metadefs/namespaces/%s/tags' % namespace_name
        headers = self._headers({'content-type': 'application/json',
                                 'X-Openstack-Append': 'True'})
        data = {
            "tags": [
                {"name": "tag4"},
                {"name": "tag5"},
                {"name": "tag6"}
            ]
        }
        response = self.api_post(path, headers=headers, json=data)
        self.assertEqual(http.CREATED, response.status_code)

        # List out all six tags.
        response = self.api_get(path, headers=self._headers())
        self.assertEqual(http.OK, response.status_code)
        tags = jsonutils.loads(response.text)['tags']
        self.assertEqual(6, len(tags))

        # Attempt to create duplicate existing tag6
        data = {
            "tags": [
                {"name": "tag6"},
                {"name": "tag7"},
                {"name": "tag8"}
            ]
        }
        response = self.api_post(path, headers=headers, json=data)
        self.assertEqual(http.CONFLICT, response.status_code)

        # Verify the previous 6 still exist
        response = self.api_get(path, headers=self._headers())
        self.assertEqual(http.OK, response.status_code)
        tags = jsonutils.loads(response.text)['tags']
        self.assertEqual(6, len(tags))

    def _create_tags(self, namespaces):
        tags = []
        for namespace in namespaces:
            headers = self._headers({'X-Tenant-Id': namespace['owner']})
            tag_name = "tag_of_%s" % (namespace['namespace'])
            path = '/v2/metadefs/namespaces/%s/tags/%s' % (
                namespace['namespace'], tag_name)
            response = self.api_post(path, headers=headers)
            self.assertEqual(http.CREATED, response.status_code)
            tag_metadata = jsonutils.loads(response.text)
            metadef_tags = dict()
            metadef_tags[namespace['namespace']] = tag_metadata['name']
            tags.append(metadef_tags)

        return tags

    def _update_tags(self, path, headers, data):
        # The tag should be mutable
        response = self.api_put(path, headers=headers, json=data)
        self.assertEqual(http.OK, response.status_code, response.text)
        # Returned metadata_tag should reflect the changes
        metadata_tag = jsonutils.loads(response.text)
        self.assertEqual(data['name'], metadata_tag['name'])

        # Updates should persist across requests
        response = self.api_get(path, headers=self._headers())
        self.assertEqual(http.OK, response.status_code)
        self.assertEqual(data['name'], metadata_tag['name'])

    def test_role_base_metadata_tags_lifecycle(self):
        # Create public and private namespaces for tenant1 and tenant2
        path = '/v2/metadefs/namespaces'
        headers = self._headers({'content-type': 'application/json'})
        tenant1_namespaces = []
        tenant2_namespaces = []
        for tenant in [self.tenant1, self.tenant2]:
            headers['X-Tenant-Id'] = tenant
            for visibility in ['public', 'private']:
                namespace_data = {
                    "namespace": "%s_%s_namespace" % (tenant, visibility),
                    "display_name": "My User Friendly Namespace",
                    "description": "My description",
                    "visibility": visibility,
                    "owner": tenant
                }
                namespace = self.create_namespace(path, headers,
                                                  namespace_data)
                namespace = jsonutils.loads(namespace.text)
                self.assertNamespacesEqual(namespace, namespace_data)
                if tenant == self.tenant1:
                    tenant1_namespaces.append(namespace)
                else:
                    tenant2_namespaces.append(namespace)

        # Create a metadef tags for each namespace created above
        tenant1_tags = self._create_tags(tenant1_namespaces)
        tenant2_tags = self._create_tags(tenant2_namespaces)

        def _check_tag_access(tags, tenant):
            headers = self._headers({'content-type': 'application/json',
                                     'X-Tenant-Id': tenant,
                                     'X-Roles': 'reader,member'})
            for tag in tags:
                for namespace, tag_name in tag.items():
                    path = '/v2/metadefs/namespaces/%s/tags/%s' % (
                        namespace, tag_name)
                    response = self.api_get(path, headers=headers)
                    if namespace.split('_')[1] == 'public':
                        expected = http.OK
                    else:
                        expected = http.NOT_FOUND

                    # Make sure we can see all public and own private tags,
                    # but not the private tags of other tenant's
                    self.assertEqual(expected, response.status_code)

                    # Make sure the same holds for listing
                    path = '/v2/metadefs/namespaces/%s/tags' % namespace
                    response = self.api_get(path, headers=headers)
                    self.assertEqual(expected, response.status_code)
                    if expected == http.OK:
                        resp_props = jsonutils.loads(response.text)['tags']
                        self.assertEqual(
                            sorted(tag.values()),
                            sorted([x['name']
                                    for x in resp_props]))

        # Check Tenant 1 can access tags of all public namespace
        # and cannot access tags of private namespace of Tenant 2
        _check_tag_access(tenant2_tags, self.tenant1)

        # Check Tenant 2 can access tags of public namespace and
        # cannot access tags of private namespace of Tenant 1
        _check_tag_access(tenant1_tags, self.tenant2)

        # Update tags with admin and non admin role
        total_tags = tenant1_tags + tenant2_tags
        for tag in total_tags:
            for namespace, tag_name in tag.items():
                data = {
                    "name": tag_name}
                path = '/v2/metadefs/namespaces/%s/tags/%s' % (
                    namespace, tag_name)

                # Update tag should fail with non admin role
                headers['X-Roles'] = "reader,member"
                response = self.api_put(path, headers=headers, json=data)
                self.assertEqual(http.FORBIDDEN, response.status_code)

                # Should work with admin role
                headers = self._headers({
                    'X-Tenant-Id': namespace.split('_')[0]})
                self._update_tags(path, headers, data)

        # Delete tags should not be allowed to non admin role
        for tag in total_tags:
            for namespace, tag_name in tag.items():
                path = '/v2/metadefs/namespaces/%s/tags/%s' % (
                    namespace, tag_name)
                response = self.api_delete(
                    path, headers=self._headers({
                        'X-Roles': 'reader,member',
                        'X-Tenant-Id': namespace.split('_')[0]
                    }))
                self.assertEqual(http.FORBIDDEN, response.status_code)

        # Delete all metadef tags
        headers = self._headers()
        for tag in total_tags:
            for namespace, tag_name in tag.items():
                path = '/v2/metadefs/namespaces/%s/tags/%s' % (
                    namespace, tag_name)
                response = self.api_delete(path, headers=headers)
                self.assertEqual(http.NO_CONTENT, response.status_code)

                # Deleted tags should not exist
                response = self.api_get(path, headers=headers)
                self.assertEqual(http.NOT_FOUND, response.status_code)

        # Create multiple tags should not be allowed to non admin role
        headers = self._headers({'content-type': 'application/json',
                                 'X-Roles': 'reader,member'})
        data = {
            "tags": [{"name": "tag1"}, {"name": "tag2"}, {"name": "tag3"}]
        }
        for namespace in tenant1_namespaces:
            path = '/v2/metadefs/namespaces/%s/tags' % (
                namespace['namespace'])
            response = self.api_post(path, headers=headers, json=data)
            self.assertEqual(http.FORBIDDEN, response.status_code)

        # Create multiple tags.
        headers = self._headers({'content-type': 'application/json'})
        for namespace in tenant1_namespaces:
            path = '/v2/metadefs/namespaces/%s/tags' % (namespace['namespace'])
            response = self.api_post(path, headers=headers, json=data)
            self.assertEqual(http.CREATED, response.status_code)

        # Delete multiple tags should not be allowed with non admin role
        headers = self._headers({'content-type': 'application/json',
                                 'X-Roles': 'reader,member'})
        for namespace in tenant1_namespaces:
            path = '/v2/metadefs/namespaces/%s/tags' % (namespace['namespace'])
            response = self.api_delete(path, headers=headers)
            self.assertEqual(http.FORBIDDEN, response.status_code)

        # Delete multiple tags created above created tags
        headers = self._headers()
        for namespace in tenant1_namespaces:
            path = '/v2/metadefs/namespaces/%s/tags' % (namespace['namespace'])
            response = self.api_delete(path, headers=headers)
            self.assertEqual(http.NO_CONTENT, response.status_code)



#!/usr/bin/env python

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# Copyright 2011 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Glance API Server
"""

import os
import sys

import eventlet
eventlet.patcher.monkey_patch()

from oslo_reports import guru_meditation_report as gmr

# If ../glance/__init__.py exists, add ../ to Python search path, so that
# it will override what happens to be installed in /usr/(local/)lib/python...
possible_topdir = os.path.normpath(os.path.join(os.path.abspath(sys.argv[0]),
                                   os.pardir,
                                   os.pardir))
if os.path.exists(os.path.join(possible_topdir, 'glance', '__init__.py')):
    sys.path.insert(0, possible_topdir)

import glance_store
from oslo_config import cfg
from oslo_log import log as logging
import osprofiler.initializer

import glance.async_
from glance.common import config
from glance.common import exception
from glance.common import wsgi
from glance import notifier
from glance import version

CONF = cfg.CONF
CONF.import_group("profiler", "glance.common.wsgi")
logging.register_options(CONF)

# NOTE(rosmaita): Any new exceptions added should preserve the current
# error codes for backward compatibility.  The value 99 is returned
# for errors not listed in this map.
ERROR_CODE_MAP = {RuntimeError: 1,
                  exception.WorkerCreationFailure: 2,
                  glance_store.exceptions.BadStoreConfiguration: 3,
                  ValueError: 4,
                  cfg.ConfigFileValueError: 5}


def fail(e):
    sys.stderr.write("ERROR: %s\n" % e)
    return_code = ERROR_CODE_MAP.get(type(e), 99)
    sys.exit(return_code)


def main():
    try:
        config.parse_args()
        config.set_config_defaults()
        wsgi.set_eventlet_hub()
        logging.setup(CONF, 'glance')
        gmr.TextGuruMeditation.setup_autorun(version)
        notifier.set_defaults()

        if CONF.profiler.enabled:
            osprofiler.initializer.init_from_conf(
                conf=CONF,
                context={},
                project="glance",
                service="api",
                host=CONF.bind_host
            )

        # NOTE(danms): Configure system-wide threading model to use eventlet
        glance.async_.set_threadpool_model('eventlet')

        server = wsgi.Server(initialize_glance_store=True)
        server.start(config.load_paste_app('glance-api'), default_port=9292)
        server.wait()
    except Exception as e:
        fail(e)


if __name__ == '__main__':
    main()



#    Copyright (c) 2023 RedHat, Inc.
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_db.sqlalchemy import test_fixtures
from oslo_db.sqlalchemy import utils as db_utils
import sqlalchemy

from glance.tests.functional.db import test_migrations
import glance.tests.utils as test_utils


class Test2024_1Expand01Mixin(test_migrations.AlembicMigrationsMixin):

    def _get_revisions(self, config):
        return test_migrations.AlembicMigrationsMixin._get_revisions(
            self, config, head='2024_1_expand01')

    def _pre_upgrade_2024_1_expand01(self, engine):
        self.assertRaises(sqlalchemy.exc.NoSuchTableError,
                          db_utils.get_table, engine, 'node_reference')
        self.assertRaises(sqlalchemy.exc.NoSuchTableError,
                          db_utils.get_table, engine, 'cached_images')

    def _check_2024_1_expand01(self, engine, data):
        # check that after migration, 'node_reference' and 'cached_images'
        # tables are created with expected columns and indexes
        node_reference = db_utils.get_table(engine, 'node_reference')
        self.assertIn('node_reference_id', node_reference.c)
        self.assertIn('node_reference_url', node_reference.c)
        self.assertTrue(db_utils.index_exists(
            engine, 'node_reference',
            'uq_node_reference_node_reference_url'),
            'Index %s on table %s does not exist' %
            ('uq_node_reference_node_reference_url', 'node_reference'))

        cached_images = db_utils.get_table(engine, 'cached_images')
        self.assertIn('id', cached_images.c)
        self.assertIn('image_id', cached_images.c)
        self.assertIn('last_accessed', cached_images.c)
        self.assertIn('last_modified', cached_images.c)
        self.assertIn('size', cached_images.c)
        self.assertIn('hits', cached_images.c)
        self.assertIn('checksum', cached_images.c)
        self.assertIn('node_reference_id', cached_images.c)
        self.assertTrue(db_utils.index_exists(
            engine, 'cached_images',
            'ix_cached_images_image_id_node_reference_id'),
            'Index %s on table %s does not exist' %
            ('ix_cached_images_image_id_node_reference_id', 'cached_images'))


class Test2024_1Expand01MySQL(
    Test2024_1Expand01Mixin,
    test_fixtures.OpportunisticDBTestMixin,
    test_utils.BaseTestCase,
):
    FIXTURE = test_fixtures.MySQLOpportunisticFixture



# Copyright 2011 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
from unittest import mock

from glance.common import timeutils
from glance.tests import utils as test_utils


class TimeUtilsTest(test_utils.BaseTestCase):

    def setUp(self):
        super(TimeUtilsTest, self).setUp()
        self.skynet_self_aware_time_str = '1997-08-29T06:14:00Z'
        self.skynet_self_aware_time_ms_str = '1997-08-29T06:14:00.000123Z'
        self.skynet_self_aware_time = datetime.datetime(1997, 8, 29, 6, 14, 0)
        self.skynet_self_aware_ms_time = datetime.datetime(
            1997, 8, 29, 6, 14, 0, 123)
        self.one_minute_before = datetime.datetime(1997, 8, 29, 6, 13, 0)
        self.one_minute_after = datetime.datetime(1997, 8, 29, 6, 15, 0)
        self.skynet_self_aware_time_perfect_str = '1997-08-29T06:14:00.000000'
        self.skynet_self_aware_time_perfect = datetime.datetime(1997, 8, 29,
                                                                6, 14, 0)

    def test_isotime(self):
        with mock.patch('oslo_utils.timeutils.utcnow') as utcnow_mock:
            utcnow_mock.return_value = self.skynet_self_aware_time
            dt = timeutils.isotime()
            self.assertEqual(dt, self.skynet_self_aware_time_str)

    def test_isotimei_micro_second_precision(self):
        with mock.patch('oslo_utils.timeutils.utcnow') as utcnow_mock:
            utcnow_mock.return_value = self.skynet_self_aware_ms_time
            dt = timeutils.isotime(subsecond=True)
            self.assertEqual(dt, self.skynet_self_aware_time_ms_str)



# Copyright 2011-2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import sys
import time

from oslo_config import cfg
from oslo_utils import units
import psutil

from glance import context
import glance.db as db_api
from glance.tests import functional
from glance.tests import utils as test_utils
from glance.tests.utils import execute

CONF = cfg.CONF


class TestScrubber(functional.SynchronousAPIBase):

    """Test that delayed_delete works and the scrubber deletes"""

    def setUp(self):
        super(TestScrubber, self).setUp()
        self.admin_context = context.get_admin_context(show_deleted=True)

    def tearDown(self):
        self._kill_scrubber_if_running()
        super(TestScrubber, self).tearDown()

    @staticmethod
    def _kill_scrubber_if_running():
        """Kill all running glance.cmd.scrubber processes"""
        for proc in psutil.process_iter(['pid', 'cmdline']):
            cmdline = proc.info['cmdline']
            if cmdline and 'glance.cmd.scrubber' in cmdline:
                try:
                    proc.terminate()
                    try:
                        proc.wait(timeout=5)
                    except Exception:
                        proc.kill()
                        proc.wait()
                except Exception:
                    # Ignore errors if not running
                    pass

    def _get_pending_delete_image(self, image_id):
        # In Glance V2, there is no way to get the 'pending_delete' image from
        # API. So we get the image from db here for testing.
        # Clean the session cache first to avoid connecting to the old db data.
        db_api.get_api().clear_db_env()
        image = db_api.get_api().image_get(self.admin_context, image_id)
        return image

    def test_delayed_delete(self):
        """
        test that images don't get deleted immediately and that the scrubber
        scrubs them
        """
        self.config(delayed_delete=True)
        self.start_server()
        self.scrubber = self.start_scrubber(daemon=True, wakeup_time=2)[3]

        data = test_utils.FakeData(1 * units.Mi)
        image_id = self._create_and_upload(data_iter=data)

        image = self.api_get('/v2/images/%s' % image_id).json
        self.assertEqual('active', image['status'])

        path = '/v2/images/%s' % image_id
        self.api_delete(path)

        image = self._get_pending_delete_image(image['id'])
        self.assertEqual('pending_delete', image['status'])

        if self.scrubber:
            self.wait_for_scrub(image['id'])

    def test_scrubber_app(self):
        """
        test that the glance-scrubber script runs successfully when not in
        daemon mode
        """
        self.config(delayed_delete=True)
        self.start_server()

        data = test_utils.FakeData(1 * units.Mi)
        image_id = self._create_and_upload(data_iter=data)

        image = self.api_get('/v2/images/%s' % image_id).json
        self.assertEqual('active', image['status'])

        path = '/v2/images/%s' % image_id
        self.api_delete(path)

        image = self._get_pending_delete_image(image['id'])
        self.assertEqual('pending_delete', image['status'])

        # scrub images and make sure they get deleted
        self.start_scrubber(daemon=False, wakeup_time=2)
        self.wait_for_scrub(image['id'])

    def test_scrubber_delete_handles_exception(self):
        """
        Test that the scrubber handles the case where an
        exception occurs when _delete() is called. The scrubber
        should not write out queue files in this case.
        """
        self.config(delayed_delete=True)
        self.start_server()

        # add an image
        data = test_utils.FakeData(1 * units.Mi)
        image_id = self._create_and_upload(data_iter=data)

        image = self.api_get('/v2/images/%s' % image_id).json
        self.assertEqual('active', image['status'])

        # delete the image
        path = '/v2/images/%s' % image_id
        self.api_delete(path)

        # ensure the image is marked pending delete.
        image = self._get_pending_delete_image(image['id'])
        self.assertEqual('pending_delete', image['status'])

        # Remove the file from the backend.
        store_path = os.path.join(self.test_dir, 'store1')
        file_path = os.path.join(store_path, image['id'])
        os.remove(file_path)

        # run the scrubber app, and ensure it doesn't fall over
        self.start_scrubber(daemon=False, wakeup_time=2)
        self.wait_for_scrub(image['id'])

    def test_scrubber_restore_image(self):
        # Ensure no scrubber process is running before test
        self._kill_scrubber_if_running()

        self.config(delayed_delete=True)
        self.start_server()

        # add an image
        data = test_utils.FakeData(1 * units.Mi)
        image_id = self._create_and_upload(data_iter=data)

        image = self.api_get('/v2/images/%s' % image_id).json
        self.assertEqual('active', image['status'])

        # delete the image
        path = '/v2/images/%s' % image_id
        self.api_delete(path)

        # ensure the image is marked pending delete.
        image = self._get_pending_delete_image(image['id'])
        self.assertEqual('pending_delete', image['status'])

        def _test_content():
            return self.start_scrubber(daemon=False, wakeup_time=2,
                                       restore=image['id'])

        exitcode, out, err = self.wait_for_scrubber_shutdown(
            _test_content)
        self.assertEqual(0, exitcode)

        image = self.api_get('/v2/images/%s' % image_id).json
        self.assertEqual('active', image['status'])

    def test_scrubber_restore_active_image_raise_error(self):
        self.config(delayed_delete=True)
        self.start_server()

        # add an image
        data = test_utils.FakeData(1 * units.Mi)
        image_id = self._create_and_upload(data_iter=data)

        image = self.api_get('/v2/images/%s' % image_id).json
        self.assertEqual('active', image['status'])

        def _test_content():
            return self.start_scrubber(daemon=False, wakeup_time=2,
                                       restore=image['id'], raise_error=False)

        exitcode, out, err = self.wait_for_scrubber_shutdown(
            _test_content)
        self.assertEqual(1, exitcode)
        self.assertIn('cannot restore the image from active to active '
                      '(wanted from_state=pending_delete)', str(err))

    def test_scrubber_restore_image_non_exist(self):
        def _test_content():
            return self.start_scrubber(
                daemon=False, wakeup_time=2, restore='fake_image_id',
                raise_error=False)

        exitcode, out, err = self.wait_for_scrubber_shutdown(
            _test_content)
        self.assertEqual(1, exitcode)
        self.assertIn('No image found with ID fake_image_id', str(err))

    def test_scrubber_restore_image_with_daemon_raise_error(self):
        exe_cmd = "%s -m glance.cmd.scrubber" % sys.executable
        cmd = ("%s --daemon --restore fake_image_id" % exe_cmd)
        exitcode, out, err = execute(cmd, raise_error=False)

        self.assertEqual(1, exitcode)
        self.assertIn('The restore and daemon options should not be set '
                      'together', str(err))

    def test_scrubber_restore_image_with_daemon_running(self):
        self.scrubber = self.start_scrubber(daemon=True, wakeup_time=2)[3]
        # Give the scrubber some time to start.
        time.sleep(5)

        exe_cmd = "%s -m glance.cmd.scrubber" % sys.executable
        cmd = ("%s --restore fake_image_id" % exe_cmd)
        exitcode, out, err = execute(cmd, raise_error=False)
        self.assertEqual(1, exitcode)
        self.assertIn('glance-scrubber is already running', str(err))

    def wait_for_scrubber_shutdown(self, func):
        # NOTE(wangxiyuan, rosmaita): The image-restore functionality contains
        # a check to make sure the scrubber isn't also running in daemon mode
        # to prevent a race condition between a delete and a restore.
        # Sometimes the glance-scrubber process which is setup by the
        # previous test can't be shutdown immediately, so if we get the "daemon
        # running" message we sleep and try again.
        not_down_msg = 'glance-scrubber is already running'
        total_wait = 15
        for _ in range(total_wait):
            exitcode, out, err = func()
            if exitcode == 1 and not_down_msg in str(err):
                time.sleep(1)
                continue
            return exitcode, out, err
        else:
            self.fail('Scrubber did not shut down within {} sec'.format(
                total_wait))

    def wait_for_scrub(self, image_id):
        """
        NOTE(jkoelker) The build servers sometimes take longer than 15 seconds
        to scrub. Give it up to 5 min, checking checking every 15 seconds.
        When/if it flips to deleted, bail immediately.
        """
        wait_for = 300    # seconds
        check_every = 15  # seconds
        for _ in range(wait_for // check_every):
            time.sleep(check_every)
            image = db_api.get_api().image_get(self.admin_context, image_id)
            if (image['status'] == 'deleted' and
                    image['deleted'] == True):
                break
            else:
                continue
        else:
            self.fail('image was never scrubbed')



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import http.client

from oslo_config import cfg
from oslo_serialization import jsonutils

from glance.tests.integration.v2 import base

CONF = cfg.CONF


class TestPropertyQuotaViolations(base.ApiTest):
    def __init__(self, *args, **kwargs):
        super(TestPropertyQuotaViolations, self).__init__(*args, **kwargs)
        self.api_flavor = 'noauth'

    def _headers(self, custom_headers=None):
        base_headers = {
            'X-Identity-Status': 'Confirmed',
            'X-Auth-Token': '932c5c84-02ac-4fe5-a9ba-620af0e2bb96',
            'X-User-Id': 'f9a41d13-0c13-47e9-bee2-ce4e8bfe958e',
            'X-Tenant-Id': "foo",
            'X-Roles': 'member',
        }
        base_headers.update(custom_headers or {})
        return base_headers

    def _get(self, image_id=""):
        path = ('/v2/images/%s' % image_id).rstrip('/')
        rsp, content = self.http.request(path, 'GET', headers=self._headers())
        self.assertEqual(http.client.OK, rsp.status)
        content = jsonutils.loads(content)
        return content

    def _create_image(self, body):
        path = '/v2/images'
        headers = self._headers({'content-type': 'application/json'})
        rsp, content = self.http.request(path, 'POST', headers=headers,
                                         body=jsonutils.dumps(body))
        self.assertEqual(http.client.CREATED, rsp.status)
        return jsonutils.loads(content)

    def _patch(self, image_id, body, expected_status):
        path = '/v2/images/%s' % image_id
        media_type = 'application/openstack-images-v2.1-json-patch'
        headers = self._headers({'content-type': media_type})
        rsp, content = self.http.request(path, 'PATCH', headers=headers,
                                         body=jsonutils.dumps(body))
        self.assertEqual(expected_status, rsp.status, content)
        return content

    def test_property_ops_when_quota_violated(self):
        # Image list must be empty to begin with
        image_list = self._get()['images']
        self.assertEqual(0, len(image_list))

        orig_property_quota = 10
        CONF.set_override('image_property_quota', orig_property_quota)

        # Create an image (with deployer-defined properties)
        req_body = {'name': 'testimg',
                    'disk_format': 'aki',
                    'container_format': 'aki'}
        for i in range(orig_property_quota):
            req_body['k_%d' % i] = 'v_%d' % i
        image = self._create_image(req_body)
        image_id = image['id']
        for i in range(orig_property_quota):
            self.assertEqual('v_%d' % i, image['k_%d' % i])

        # Now reduce property quota. We should be allowed to modify/delete
        # existing properties (even if the result still exceeds property quota)
        # but not add new properties nor replace existing properties with new
        # properties (as long as we're over the quota)
        self.config(image_property_quota=2)

        patch_body = [{'op': 'replace', 'path': '/k_4', 'value': 'v_4.new'}]
        image = jsonutils.loads(self._patch(image_id, patch_body,
                                            http.client.OK))
        self.assertEqual('v_4.new', image['k_4'])

        patch_body = [{'op': 'remove', 'path': '/k_7'}]
        image = jsonutils.loads(self._patch(image_id, patch_body,
                                            http.client.OK))
        self.assertNotIn('k_7', image)

        patch_body = [{'op': 'add', 'path': '/k_100', 'value': 'v_100'}]
        self._patch(image_id, patch_body, http.client.REQUEST_ENTITY_TOO_LARGE)
        image = self._get(image_id)
        self.assertNotIn('k_100', image)

        patch_body = [
            {'op': 'remove', 'path': '/k_5'},
            {'op': 'add', 'path': '/k_100', 'value': 'v_100'},
        ]
        self._patch(image_id, patch_body, http.client.REQUEST_ENTITY_TOO_LARGE)
        image = self._get(image_id)
        self.assertNotIn('k_100', image)
        self.assertIn('k_5', image)

        # temporary violations to property quota should be allowed as long as
        # it's within one PATCH request and the end result does not violate
        # quotas.
        patch_body = [{'op': 'add', 'path': '/k_100', 'value': 'v_100'},
                      {'op': 'add', 'path': '/k_99', 'value': 'v_99'}]
        to_rm = ['k_%d' % i for i in range(orig_property_quota) if i != 7]
        patch_body.extend([{'op': 'remove', 'path': '/%s' % k} for k in to_rm])
        image = jsonutils.loads(self._patch(image_id, patch_body,
                                            http.client.OK))
        self.assertEqual('v_99', image['k_99'])
        self.assertEqual('v_100', image['k_100'])
        for k in to_rm:
            self.assertNotIn(k, image)



# Copyright 2015 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import glance_store as store
import webob

import glance.api.v2.image_actions as image_actions
import glance.context
from glance.tests.unit import base
import glance.tests.unit.utils as unit_test_utils


BASE_URI = unit_test_utils.BASE_URI

USER1 = '54492ba0-f4df-4e4e-be62-27f4d76b29cf'
UUID1 = 'c80a1a6c-bd1f-41c5-90ee-81afedb1d58d'
TENANT1 = '6838eb7b-6ded-434a-882c-b344c77fe8df'
CHKSUM = '93264c3edf5972c9f1cb309543d38a5c'


def _db_fixture(id, **kwargs):
    obj = {
        'id': id,
        'name': None,
        'visibility': 'shared',
        'properties': {},
        'checksum': None,
        'owner': None,
        'status': 'queued',
        'tags': [],
        'size': None,
        'virtual_size': None,
        'locations': [],
        'protected': False,
        'disk_format': None,
        'container_format': None,
        'deleted': False,
        'min_ram': None,
        'min_disk': None,
    }
    obj.update(kwargs)
    return obj


class TestImageActionsController(base.IsolatedUnitTest):
    def setUp(self):
        super(TestImageActionsController, self).setUp()
        self.db = unit_test_utils.FakeDB(initialize=False)
        self.policy = unit_test_utils.FakePolicyEnforcer()
        self.notifier = unit_test_utils.FakeNotifier()
        self.store = unit_test_utils.FakeStoreAPI()
        for i in range(1, 4):
            self.store.data['%s/fake_location_%i' % (BASE_URI, i)] = ('Z', 1)
        self.store_utils = unit_test_utils.FakeStoreUtils(self.store)
        self.controller = image_actions.ImageActionsController(
            self.db,
            self.policy,
            self.notifier,
            self.store)
        self.controller.gateway.store_utils = self.store_utils
        store.create_stores()

    def _get_fake_context(self, user=USER1, tenant=TENANT1, roles=None,
                          is_admin=False):
        if roles is None:
            roles = ['member']

        kwargs = {
            'user': user,
            'tenant': tenant,
            'roles': roles,
            'is_admin': is_admin,
        }

        context = glance.context.RequestContext(**kwargs)
        return context

    def _create_image(self, status):
        self.images = [
            _db_fixture(UUID1, owner=TENANT1, checksum=CHKSUM,
                        name='1', size=256, virtual_size=1024,
                        visibility='public',
                        locations=[{'url': '%s/%s' % (BASE_URI, UUID1),
                                    'metadata': {}, 'status': 'active'}],
                        disk_format='raw',
                        container_format='bare',
                        status=status),
        ]
        context = self._get_fake_context()
        [self.db.image_create(context, image) for image in self.images]

    def test_deactivate_from_active(self):
        self._create_image('active')

        request = unit_test_utils.get_fake_request()
        self.controller.deactivate(request, UUID1)

        image = self.db.image_get(request.context, UUID1)

        self.assertEqual('deactivated', image['status'])

    def test_deactivate_from_deactivated(self):
        self._create_image('deactivated')

        request = unit_test_utils.get_fake_request()
        self.controller.deactivate(request, UUID1)

        image = self.db.image_get(request.context, UUID1)

        self.assertEqual('deactivated', image['status'])

    def _test_deactivate_from_wrong_status(self, status):

        # deactivate will yield an error if the initial status is anything
        # other than 'active' or 'deactivated'
        self._create_image(status)

        request = unit_test_utils.get_fake_request()
        self.assertRaises(webob.exc.HTTPForbidden, self.controller.deactivate,
                          request, UUID1)

    def test_deactivate_from_queued(self):
        self._test_deactivate_from_wrong_status('queued')

    def test_deactivate_from_saving(self):
        self._test_deactivate_from_wrong_status('saving')

    def test_deactivate_from_killed(self):
        self._test_deactivate_from_wrong_status('killed')

    def test_deactivate_from_pending_delete(self):
        self._test_deactivate_from_wrong_status('pending_delete')

    def test_deactivate_from_deleted(self):
        self._test_deactivate_from_wrong_status('deleted')

    def test_reactivate_from_active(self):
        self._create_image('active')

        request = unit_test_utils.get_fake_request()
        self.controller.reactivate(request, UUID1)

        image = self.db.image_get(request.context, UUID1)

        self.assertEqual('active', image['status'])

    def test_reactivate_from_deactivated(self):
        self._create_image('deactivated')

        request = unit_test_utils.get_fake_request()
        self.controller.reactivate(request, UUID1)

        image = self.db.image_get(request.context, UUID1)

        self.assertEqual('active', image['status'])

    def _test_reactivate_from_wrong_status(self, status):

        # reactivate will yield an error if the initial status is anything
        # other than 'active' or 'deactivated'
        self._create_image(status)

        request = unit_test_utils.get_fake_request()
        self.assertRaises(webob.exc.HTTPForbidden, self.controller.reactivate,
                          request, UUID1)

    def test_reactivate_from_queued(self):
        self._test_reactivate_from_wrong_status('queued')

    def test_reactivate_from_saving(self):
        self._test_reactivate_from_wrong_status('saving')

    def test_reactivate_from_killed(self):
        self._test_reactivate_from_wrong_status('killed')

    def test_reactivate_from_pending_delete(self):
        self._test_reactivate_from_wrong_status('pending_delete')

    def test_reactivate_from_deleted(self):
        self._test_reactivate_from_wrong_status('deleted')



# Copyright 2013 Rackspace Hosting
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import atexit
import os.path
import shutil
import tempfile

import fixtures
import glance_store
from oslo_config import cfg
from oslo_db import options

import glance.common.client
from glance.common import config
import glance.db.sqlalchemy.api
from glance import tests as glance_tests
from glance.tests import utils as test_utils


TESTING_API_PASTE_CONF = """
[composite:glance-api]
paste.composite_factory = glance.api:root_app_factory
/: api

[pipeline: api]
pipeline = versionnegotiation gzip unauthenticated-context rootapp

[composite:glance-api-caching]
paste.composite_factory = glance.api:root_app_factory
/: api-caching

[pipeline: api-caching]
pipeline = versionnegotiation gzip unauthenticated-context cache rootapp

[composite:glance-api-cachemanagement]
paste.composite_factory = glance.api:root_app_factory
/: api-cachemanagement

[pipeline: api-cachemanagement]
pipeline =
    versionnegotiation
    gzip
    unauthenticated-context
    cache
    cache_manage
    rootapp

[composite:glance-api-fakeauth]
paste.composite_factory = glance.api:root_app_factory
/: api-fakeauth

[pipeline: api-fakeauth]
pipeline = versionnegotiation gzip fakeauth context rootapp

[composite:glance-api-noauth]
paste.composite_factory = glance.api:root_app_factory
/: api-noauth

[pipeline: api-noauth]
pipeline = versionnegotiation gzip context rootapp

[composite:rootapp]
paste.composite_factory = glance.api:root_app_factory
/: apiversions
/v2: apiv2app

[app:apiversions]
paste.app_factory = glance.api.versions:create_resource

[app:apiv2app]
paste.app_factory = glance.api.v2.router:API.factory

[filter:versionnegotiation]
paste.filter_factory =
 glance.api.middleware.version_negotiation:VersionNegotiationFilter.factory

[filter:gzip]
paste.filter_factory = glance.api.middleware.gzip:GzipMiddleware.factory

[filter:cache]
paste.filter_factory = glance.api.middleware.cache:CacheFilter.factory

[filter:cache_manage]
paste.filter_factory =
 glance.api.middleware.cache_manage:CacheManageFilter.factory

[filter:context]
paste.filter_factory = glance.api.middleware.context:ContextMiddleware.factory

[filter:unauthenticated-context]
paste.filter_factory =
 glance.api.middleware.context:UnauthenticatedContextMiddleware.factory

[filter:fakeauth]
paste.filter_factory = glance.tests.utils:FakeAuthMiddleware.factory
"""

CONF = cfg.CONF


class ApiTest(test_utils.BaseTestCase):

    def setUp(self):
        super(ApiTest, self).setUp()
        self.test_dir = self.useFixture(fixtures.TempDir()).path
        self._configure_logging()
        self._setup_database()
        self._setup_stores()
        self._setup_property_protection()
        self.glance_api_app = self._load_paste_app(
            'glance-api',
            flavor=getattr(self, 'api_flavor', ''),
            conf=getattr(self, 'api_paste_conf', TESTING_API_PASTE_CONF),
        )
        self.http = test_utils.Httplib2WsgiAdapter(self.glance_api_app)

    def _setup_property_protection(self):
        self._copy_data_file('property-protections.conf', self.test_dir)
        self.property_file = os.path.join(self.test_dir,
                                          'property-protections.conf')

    def _configure_logging(self):
        self.config(default_log_levels=[
            'amqplib=WARN',
            'sqlalchemy=WARN',
            'boto=WARN',
            'suds=INFO',
            'keystone=INFO',
            'eventlet.wsgi.server=DEBUG'
        ])

    def _setup_database(self):
        sql_connection = 'sqlite:////%s/tests.sqlite' % self.test_dir
        options.set_defaults(CONF, connection=sql_connection)
        glance.db.sqlalchemy.api.clear_db_env()
        glance_db_env = 'GLANCE_DB_TEST_SQLITE_FILE'
        if glance_db_env in os.environ:
            # use the empty db created and cached as a tempfile
            # instead of spending the time creating a new one
            db_location = os.environ[glance_db_env]
            shutil.copyfile(db_location, "%s/tests.sqlite" % self.test_dir)
        else:
            test_utils.db_sync()

            # copy the clean db to a temp location so that it
            # can be reused for future tests
            (osf, db_location) = tempfile.mkstemp()
            os.close(osf)
            shutil.copyfile('%s/tests.sqlite' % self.test_dir, db_location)
            os.environ[glance_db_env] = db_location

            # cleanup the temp file when the test suite is
            # complete
            def _delete_cached_db():
                try:
                    os.remove(os.environ[glance_db_env])
                except Exception:
                    glance_tests.logger.exception(
                        "Error cleaning up the file %s" %
                        os.environ[glance_db_env])
            atexit.register(_delete_cached_db)

    def _setup_stores(self):
        glance_store.register_opts(CONF)

        image_dir = os.path.join(self.test_dir, "images")
        self.config(group='glance_store',
                    filesystem_store_datadir=image_dir)

        glance_store.create_stores()

    def _load_paste_app(self, name, flavor, conf):
        conf_file_path = os.path.join(self.test_dir, '%s-paste.ini' % name)
        with open(conf_file_path, 'w') as conf_file:
            conf_file.write(conf)
            conf_file.flush()
        return config.load_paste_app(name, flavor=flavor,
                                     conf_file=conf_file_path)

    def tearDown(self):
        glance.db.sqlalchemy.api.clear_db_env()
        super(ApiTest, self).tearDown()



# Copyright (c) 2017 RedHat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

import glance_store as g_store
from oslo_config import cfg
from oslo_log import log as logging
import oslo_serialization.jsonutils as json
import webob.exc

from glance.api import policy
from glance.api.v2 import policy as api_policy
from glance.common import exception
from glance.common import wsgi
import glance.db
from glance.i18n import _, _LW
from glance.quota import keystone as ks_quota

CONF = cfg.CONF

LOG = logging.getLogger(__name__)


class InfoController(object):
    def __init__(self, policy_enforcer=None):
        self.policy = policy_enforcer or policy.Enforcer()

    def get_image_import(self, req):
        # TODO(jokke): All the rest of the boundaries should be implemented.
        import_methods = {
            'description': 'Import methods available.',
            'type': 'array',
            'value': CONF.get('enabled_import_methods')
        }

        return {
            'import-methods': import_methods
        }

    def get_stores(self, req):
        # TODO(abhishekk): This will be removed after config options
        # 'stores' and 'default_store' are removed.
        enabled_backends = CONF.enabled_backends
        if not enabled_backends:
            msg = _("Multi backend is not supported at this site.")
            raise webob.exc.HTTPNotFound(explanation=msg)

        backends = []
        for backend in enabled_backends:
            if backend.startswith("os_glance_"):
                continue

            stores = {}
            if enabled_backends[backend] == 'swift':
                conf_file = getattr(CONF, backend).swift_store_config_file
                multitenant = getattr(CONF, backend).swift_store_multi_tenant
                if multitenant and conf_file:
                    msg = ("The config options 'swift_store_multi_tenant' "
                           "and 'swift_store_config_file' are mutually "
                           "exclusive. If you intend to use multi-tenant "
                           "swift store, please make sure that you have "
                           "not set a swift configuration file with the "
                           "'swift_store_config_file' option. "
                           "Excluding `%s:%s` store details from the "
                           "response as it's not configured correctly."
                           % (backend, enabled_backends[backend]))
                    LOG.warning(_LW(msg))
                    continue
            stores['id'] = backend
            description = getattr(CONF, backend).store_description
            if description:
                stores['description'] = description
            if backend == CONF.glance_store.default_backend:
                stores['default'] = "true"
            # Check if http store is configured then mark it as read-only
            if enabled_backends[backend] == 'http':
                stores['read-only'] = "true"
            backends.append(stores)

        return {'stores': backends}

    @staticmethod
    def _get_fsid_from_url(store_detail):
        fsid = None
        prefix = 'rbd://'
        url_prefix = store_detail.url_prefix
        # When fsid and pool info are not available,
        # url_prefix is same as prefix
        if url_prefix and url_prefix.startswith(
                prefix) and len(url_prefix) > len(prefix):
            # Remove last trailing forward slash
            url_prefix = (
                url_prefix[:-1] if url_prefix.endswith('/') else url_prefix)
            pieces = url_prefix[len(prefix):].split('/')
            # We expect the rbd store's url format to look like 'rbd://%s/%s/'
            # where the fsid is in the first position; if there are more than
            # 2 pieces, then something has changed in the driver code, so we
            # won't set the fsid
            if len(pieces) == 2:
                fsid = pieces[0]
        return fsid

    @staticmethod
    def _get_rbd_properties(store_detail):
        return {
            'chunk_size': store_detail.chunk_size,
            'pool': store_detail.pool,
            'thin_provisioning': store_detail.thin_provisioning,
            'fsid': InfoController._get_fsid_from_url(store_detail),
        }

    @staticmethod
    def _get_file_properties(store_detail):
        return {
            'data_dir': store_detail.datadir,
            'chunk_size': store_detail.chunk_size,
            'thin_provisioning': store_detail.thin_provisioning
        }

    @staticmethod
    def _get_cinder_properties(store_detail):
        return {
            'volume_type': store_detail.store_conf.cinder_volume_type,
            'use_multipath': store_detail.store_conf.cinder_use_multipath
        }

    @staticmethod
    def _get_swift_properties(store_detail):
        return {
            'container': getattr(store_detail, 'container', None),
            'large_object_size': store_detail.large_object_size,
            'large_object_chunk_size': store_detail.large_object_chunk_size
        }

    @staticmethod
    def _get_s3_properties(store_detail):
        return {
            's3_store_large_object_size':
            store_detail.s3_store_large_object_size,
            's3_store_large_object_chunk_size':
            store_detail.s3_store_large_object_chunk_size,
            's3_store_thread_pools':
            store_detail.s3_store_thread_pools
        }

    @staticmethod
    def _get_http_properties(store_detail):
        # NOTE(mrjoshi): Thre are no useful properties
        # to be exposed.
        return {}

    def get_stores_detail(self, req):
        enabled_backends = CONF.enabled_backends
        stores = self.get_stores(req).get('stores')
        try:
            api_policy.DiscoveryAPIPolicy(
                req.context,
                enforcer=self.policy).stores_info_detail()

            store_mapper = {
                'rbd': self._get_rbd_properties,
                'file': self._get_file_properties,
                'cinder': self._get_cinder_properties,
                'swift': self._get_swift_properties,
                's3': self._get_s3_properties,
                'http': self._get_http_properties
            }

            for store in stores:
                store_type = enabled_backends[store['id']]
                store['type'] = store_type
                store_detail = g_store.get_store_from_store_identifier(
                    store['id'])
                store['properties'] = store_mapper.get(store_type)(
                    store_detail)
                store['weight'] = getattr(CONF, store['id']).weight

        except exception.Forbidden as e:
            LOG.debug("User not permitted to view details")
            raise webob.exc.HTTPForbidden(explanation=e.msg)

        return {'stores': stores}

    def get_usage(self, req):
        project_usage = ks_quota.get_usage(req.context)
        return {'usage':
                {name: {'usage': usage.usage,
                        'limit': usage.limit}
                 for name, usage in project_usage.items()}}


class ResponseSerializer(wsgi.JSONResponseSerializer):
    def __init__(self, usage_schema=None):
        super(ResponseSerializer, self).__init__()
        self.schema = usage_schema or get_usage_schema()

    def get_usage(self, response, usage):
        body = json.dumps(self.schema.filter(usage), ensure_ascii=False)
        response.unicode_body = str(body)
        response.content_type = 'application/json'


_USAGE_SCHEMA = {
    'usage': {
        'type': 'array',
        'items': {
            'type': 'object',
            'additionalProperties': True,
            'validation_data': {
                'type': 'object',
                'additionalProperties': False,
                'properties': {
                    'usage': {'type': 'integer'},
                    'limit': {'type': 'integer'},
                },
            },
        },
    },
}


def get_usage_schema():
    return glance.schema.Schema('usage', copy.deepcopy(_USAGE_SCHEMA))


def create_resource():
    usage_schema = get_usage_schema()
    serializer = ResponseSerializer(usage_schema)
    return wsgi.Resource(InfoController(), None, serializer)



# Copyright 2024 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import datetime
import errno
import io
import os
import time
from unittest import mock

from oslo_utils import fileutils

from glance.image_cache.drivers import centralized_db
from glance.tests import functional


DATA = b'IMAGEDATA'


class TestCentralizedDb(functional.SynchronousAPIBase):
    # ToDo(abhishekk): Once system scope is enabled and RBAC is fully
    # supported, enable these tests for RBAC as well
    def setUp(self):
        super(TestCentralizedDb, self).setUp()

    def start_server(self, enable_cache=True, set_worker_url=True):
        if set_worker_url:
            self.config(worker_self_reference_url='http://workerx')
            self.config(image_cache_driver='centralized_db')

        super(TestCentralizedDb, self).start_server(enable_cache=enable_cache)

    def load_data(self):
        output = {}
        # Create 1 queued image as well for testing
        path = "/v2/images"
        data = {
            'name': 'queued-image',
            'container_format': 'bare',
            'disk_format': 'raw'
        }
        response = self.api_post(path, json=data)
        self.assertEqual(201, response.status_code)
        image_id = response.json['id']
        output['queued'] = image_id

        for visibility in ['public', 'private']:
            data = {
                'name': '%s-image' % visibility,
                'visibility': visibility,
                'container_format': 'bare',
                'disk_format': 'raw'
            }
            response = self.api_post(path, json=data)
            self.assertEqual(201, response.status_code)
            image_id = response.json['id']
            # Upload some data to image
            response = self.api_put(
                '/v2/images/%s/file' % image_id,
                headers={'Content-Type': 'application/octet-stream'},
                data=DATA)
            self.assertEqual(204, response.status_code)
            output[visibility] = image_id

        return output

    def wait_for_caching(self, image_id, max_sec=10, delay_sec=0.2,
                         start_delay_sec=None):
        start_time = time.time()
        done_time = start_time + max_sec
        if start_delay_sec:
            time.sleep(start_delay_sec)
        while time.time() <= done_time:
            output = self.list_cache()['cached_images']
            output = [image['image_id'] for image in output]
            if output and image_id in output:
                return
            time.sleep(delay_sec)

        msg = "Image {0} failed to cached within {1} sec"
        raise Exception(msg.format(image_id, max_sec))

    def list_cache(self, expected_code=200):
        path = '/v2/cache'
        response = self.api_get(path)
        self.assertEqual(expected_code, response.status_code)
        if response.status_code == 200:
            return response.json

    def test_centralized_db_worker_url_not_set(self):
        try:
            self.config(image_cache_driver='centralized_db')
            self.start_server(enable_cache=True, set_worker_url=False)
        except RuntimeError as e:
            expected_message = "'worker_self_reference_url' needs to be set " \
                               "if `centralized_db` is defined as cache " \
                               "driver for image_cache_driver config option."
            self.assertIn(expected_message, e.args)

    def test_centralized_db_verify_worker_node_is_set(self):
        self.start_server(enable_cache=True)
        self.driver = centralized_db.Driver()
        self.assertEqual(
            'http://workerx', self.driver.db_api.node_reference_get_by_url(
                self.driver.context, 'http://workerx').node_reference_url)

    def test_get_cache_size(self):
        self.start_server(enable_cache=True)
        images = self.load_data()
        self.driver = centralized_db.Driver()
        self.driver.configure()
        # Verify initially cache size is 0
        self.assertEqual(0, self.driver.get_cache_size())

        # Cache one image
        path = '/v2/cache/%s' % images['public']
        self.api_put(path)
        self.wait_for_caching(images['public'])

        # Verify cache size is equal to len(DATA) i.e. 9
        self.assertEqual(len(DATA), self.driver.get_cache_size())

    def test_get_hit_count(self):
        self.start_server(enable_cache=True)
        images = self.load_data()
        self.driver = centralized_db.Driver()
        self.driver.configure()
        # Verify that hit count is currently 0 as no image is cached
        self.assertEqual(0, self.driver.get_hit_count(images['public']))

        # Cache one image
        path = '/v2/cache/%s' % images['public']
        self.api_put(path)
        self.wait_for_caching(images['public'])
        # verify image is cached
        self.assertTrue(self.driver.is_cached(images['public']))

        # Verify that hit count is still 0 as image is cached, but
        # not downloaded yet from cache
        self.assertEqual(0, self.driver.get_hit_count(images['public']))

        # Download the image
        path = '/v2/images/%s/file' % images['public']
        response = self.api_get(path)
        self.assertEqual('IMAGEDATA', response.text)
        # Verify that hit count is 1 as we hit the cache
        self.assertEqual(1, self.driver.get_hit_count(images['public']))

        # Download the image again
        path = '/v2/images/%s/file' % images['public']
        response = self.api_get(path)
        self.assertEqual('IMAGEDATA', response.text)
        # Verify that hit count is 2 as we hit the cache
        self.assertEqual(2, self.driver.get_hit_count(images['public']))

    def test_get_cached_images(self):
        self.start_server(enable_cache=True)
        images = self.load_data()
        self.driver = centralized_db.Driver()
        self.driver.configure()
        # Verify that initially there are no cached image(s)
        self.assertEqual(0, len(self.driver.get_cached_images()))

        # Cache one image
        path = '/v2/cache/%s' % images['public']
        self.api_put(path)
        self.wait_for_caching(images['public'])
        # verify image is cached
        self.assertTrue(self.driver.is_cached(images['public']))

        # Verify that there is one cached imgae now
        self.assertEqual(1, len(self.driver.get_cached_images()))

        # Verify that passing non-existing node will be
        # returned as 0 cached images
        self.config(worker_self_reference_url="http://fake-worker")
        self.assertEqual(0, len(self.driver.get_cached_images()))

    def test_is_cacheable(self):
        self.start_server(enable_cache=True)
        images = self.load_data()
        self.driver = centralized_db.Driver()
        self.driver.configure()
        # Verify that is_cacheable will return true as image is not cached yet
        self.assertTrue(self.driver.is_cacheable(images['public']))

        # Now cache the image
        path = '/v2/cache/%s' % images['public']
        self.api_put(path)
        self.wait_for_caching(images['public'])
        # verify image is cached
        self.assertTrue(self.driver.is_cached(images['public']))

        # Verify that now above image is not cachable
        self.assertFalse(self.driver.is_cacheable(images['public']))

    def test_is_being_cached(self):
        self.start_server(enable_cache=True)
        images = self.load_data()
        self.driver = centralized_db.Driver()
        self.driver.configure()
        # Verify that is_being_cached will return False as
        # image is not cached yet
        self.assertFalse(self.driver.is_being_cached(images['public']))

    def test_is_queued(self):
        self.start_server(enable_cache=True)
        images = self.load_data()
        self.driver = centralized_db.Driver()
        self.driver.configure()
        # Verify that is_queued will return False as
        # image is not queued for caching yet
        self.assertFalse(self.driver.is_queued(images['public']))

        # Now queue image for caching
        path = '/v2/cache/%s' % images['public']
        self.api_put(path)
        self.assertTrue(self.driver.is_queued(images['public']))

    def test_delete_cached_image(self):
        self.start_server(enable_cache=True)
        images = self.load_data()
        self.driver = centralized_db.Driver()
        self.driver.configure()
        # Verify deleting non-existing image from cache will not fail
        self.driver.delete_cached_image('fake-image-id')

        # Now cache the image
        path = '/v2/cache/%s' % images['public']
        self.api_put(path)
        self.wait_for_caching(images['public'])
        # verify image is cached
        self.assertTrue(self.driver.is_cached(images['public']))
        self.assertEqual(1, len(self.driver.get_cached_images()))

        # Delete the image from cache
        self.driver.delete_cached_image(images['public'])

        # Verify image is deleted from cache
        self.assertFalse(self.driver.is_cached(images['public']))
        self.assertEqual(0, len(self.driver.get_cached_images()))

    def test_delete_all_cached_images(self):
        self.start_server(enable_cache=True)
        images = self.load_data()
        self.driver = centralized_db.Driver()
        self.driver.configure()
        # Verify no image is cached yet
        self.assertEqual(0, len(self.driver.get_cached_images()))

        # Verify delete call should not fail even if no images are cached
        self.driver.delete_all_cached_images()

        # Now cache the image
        path = '/v2/cache/%s' % images['public']
        self.api_put(path)
        self.wait_for_caching(images['public'])
        # verify image is cached
        self.assertTrue(self.driver.is_cached(images['public']))
        self.assertEqual(1, len(self.driver.get_cached_images()))

        # Now cache another image
        path = '/v2/cache/%s' % images['private']
        self.api_put(path)
        self.wait_for_caching(images['private'])
        # verify image is cached
        self.assertTrue(self.driver.is_cached(images['private']))
        self.assertEqual(2, len(self.driver.get_cached_images()))

        # Delete all the images form cache
        self.driver.delete_all_cached_images()

        # Verify images are deleted from cache
        self.assertEqual(0, len(self.driver.get_cached_images()))

    def test_delete_queued_image(self):
        self.start_server(enable_cache=True)
        images = self.load_data()
        self.driver = centralized_db.Driver()
        self.driver.configure()
        # Verify deleting non-existing image from queued dir will not fail
        self.driver.delete_queued_image('fake-image-id')

        # Now queue imgae for caching
        path = '/v2/cache/%s' % images['public']
        self.api_put(path)
        # verify image is queued
        self.assertTrue(self.driver.is_queued(images['public']))
        self.assertEqual(1, len(self.driver.get_queued_images()))

        # Delete the image from queued dir
        self.driver.delete_queued_image(images['public'])

        # Verify image is deleted from cache
        self.assertFalse(self.driver.is_queued(images['public']))
        self.assertEqual(0, len(self.driver.get_queued_images()))

    def test_delete_all_queued_images(self):
        self.start_server(enable_cache=True)
        images = self.load_data()
        self.driver = centralized_db.Driver()
        self.driver.configure()
        # Verify no image is cached yet
        self.assertEqual(0, len(self.driver.get_queued_images()))

        # Verify delete call should not fail even if no images are queued
        self.driver.delete_all_queued_images()

        # Now queue the image
        path = '/v2/cache/%s' % images['public']
        self.api_put(path)
        # verify image is queued
        self.assertTrue(self.driver.is_queued(images['public']))
        self.assertEqual(1, len(self.driver.get_queued_images()))

        # Now queue another image
        path = '/v2/cache/%s' % images['private']
        self.api_put(path)
        # verify image is queued
        self.assertTrue(self.driver.is_queued(images['private']))
        self.assertEqual(2, len(self.driver.get_queued_images()))

        # Delete all the images form queued dir
        self.driver.delete_all_queued_images()

        # Verify images are deleted from cache
        self.assertEqual(0, len(self.driver.get_queued_images()))

    def test_clean(self):
        self.start_server(enable_cache=True)
        self.driver = centralized_db.Driver()
        self.driver.configure()
        cache_dir = os.path.join(self.test_dir, 'cache')
        incomplete_file_path = os.path.join(cache_dir, 'incomplete', '1')
        incomplete_file = open(incomplete_file_path, 'wb')
        incomplete_file.write(DATA)
        incomplete_file.close()

        self.assertTrue(os.path.exists(incomplete_file_path))

        self.driver.clean(stall_time=0)

        self.assertFalse(os.path.exists(incomplete_file_path))

    def _test_clean_stall_time(
            self, stall_time=None, days=2, stall_failed=False):
        """
        Test the clean method removes the stalled images as expected
        """
        self.start_server(enable_cache=True)
        self.driver = centralized_db.Driver()
        self.driver.configure()

        cache_dir = os.path.join(self.test_dir, 'cache')
        incomplete_file_path_1 = os.path.join(cache_dir,
                                              'incomplete', '1')
        incomplete_file_path_2 = os.path.join(cache_dir,
                                              'incomplete', '2')

        for f in (incomplete_file_path_1, incomplete_file_path_2):
            incomplete_file = open(f, 'wb')
            incomplete_file.write(DATA)
            incomplete_file.close()

        mtime = os.path.getmtime(incomplete_file_path_1)
        pastday = (datetime.datetime.fromtimestamp(mtime) -
                   datetime.timedelta(days=days))
        atime = int(time.mktime(pastday.timetuple()))
        mtime = atime
        os.utime(incomplete_file_path_1, (atime, mtime))

        self.assertTrue(os.path.exists(incomplete_file_path_1))
        self.assertTrue(os.path.exists(incomplete_file_path_2))

        # If stall_time is None then it will wait for default time
        # of `image_cache_stall_time` which is 24 hours
        if stall_failed:
            with mock.patch.object(
                    fileutils, 'delete_if_exists') as mock_delete:
                mock_delete.side_effect = OSError(errno.ENOENT, '')
                self.driver.clean(stall_time=stall_time)
                self.assertTrue(os.path.exists(incomplete_file_path_1))
        else:
            self.driver.clean(stall_time=stall_time)
            self.assertFalse(os.path.exists(incomplete_file_path_1))

        self.assertTrue(os.path.exists(incomplete_file_path_2))

    def test_clean_stalled_none_stall_time(self):
        self._test_clean_stall_time()

    def test_clean_stalled_nonzero_stall_time(self):
        """Test the clean method removes expected images."""
        self._test_clean_stall_time(stall_time=3600, days=1)

    def test_clean_stalled_fails(self):
        """Test the clean method fails to delete file, ignores the failure"""
        self._test_clean_stall_time(stall_time=3600, days=1,
                                    stall_failed=True)

    def test_least_recently_accessed(self):
        self.start_server(enable_cache=True)
        images = self.load_data()
        self.driver = centralized_db.Driver()
        self.driver.configure()

        # Verify no image is cached yet
        self.assertEqual(0, len(self.driver.get_cached_images()))

        # Verify delete call should not fail even if no images are cached
        self.driver.delete_all_cached_images()

        # Now cache the image
        path = '/v2/cache/%s' % images['public']
        self.api_put(path)
        self.wait_for_caching(images['public'])
        # verify image is cached
        self.assertTrue(self.driver.is_cached(images['public']))
        self.assertEqual(1, len(self.driver.get_cached_images()))

        # Now cache another image
        path = '/v2/cache/%s' % images['private']
        self.api_put(path)
        self.wait_for_caching(images['private'])
        # verify image is cached
        self.assertTrue(self.driver.is_cached(images['private']))
        self.assertEqual(2, len(self.driver.get_cached_images()))

        # Verify that 1st image will be returned
        image_id, size = self.driver.get_least_recently_accessed()
        self.assertEqual(images['public'], image_id)
        self.assertEqual(len(DATA), size)

    def test_open_for_write_good(self):
        """
        Test to see if open_for_write works in normal case
        """
        self.start_server(enable_cache=True)
        self.driver = centralized_db.Driver()
        self.driver.configure()

        # test a good case
        image_id = '1'
        self.assertFalse(self.driver.is_cached(image_id))
        with self.driver.open_for_write(image_id) as cache_file:
            cache_file.write(b'a')
        self.assertTrue(self.driver.is_cached(image_id),
                        "Image %s was NOT cached!" % image_id)

        # make sure it has tidied up
        cache_dir = os.path.join(self.test_dir, 'cache')
        incomplete_file_path = os.path.join(cache_dir,
                                            'incomplete', image_id)
        cache_file_path = os.path.join(cache_dir, image_id)
        invalid_file_path = os.path.join(cache_dir, 'invalid', image_id)
        self.assertFalse(os.path.exists(incomplete_file_path))
        self.assertFalse(os.path.exists(invalid_file_path))
        self.assertTrue(os.path.exists(cache_file_path))

    def test_open_for_write_with_exception(self):
        """
        Test to see if open_for_write works in a failure case for each driver
        This case is where an exception is raised while the file is being
        written. The image is partially filled in cache and filling won't
        resume so verify the image is moved to invalid/ directory
        """
        # test a case where an exception is raised while the file is open

        self.start_server(enable_cache=True)
        self.driver = centralized_db.Driver()
        self.driver.configure()

        image_id = '1'
        self.assertFalse(self.driver.is_cached(image_id))
        try:
            with self.driver.open_for_write(image_id):
                raise IOError
        except Exception as e:
            self.assertIsInstance(e, IOError)
        self.assertFalse(self.driver.is_cached(image_id),
                         "Image %s was cached!" % image_id)
        # make sure it has tidied up
        cache_dir = os.path.join(self.test_dir, 'cache')
        incomplete_file_path = os.path.join(cache_dir,
                                            'incomplete', image_id)
        invalid_file_path = os.path.join(cache_dir, 'invalid', image_id)
        self.assertFalse(os.path.exists(incomplete_file_path))
        self.assertTrue(os.path.exists(invalid_file_path))

    def test_open_for_read_good(self):
        self.start_server(enable_cache=True)
        self.driver = centralized_db.Driver()
        self.driver.configure()
        images = self.load_data()

        self.assertFalse(self.driver.is_cached(images['public']))
        # Cache one image
        path = '/v2/cache/%s' % images['public']
        self.api_put(path)
        self.wait_for_caching(images['public'])
        # verify image is cached
        self.assertTrue(self.driver.is_cached(images['public']))
        # verify cache hit count for above image is 0
        self.assertEqual(0, self.driver.get_hit_count(images['public']))
        # Open image for read
        buff = io.BytesIO()
        with self.driver.open_for_read(images['public']) as cache_file:
            for chunk in cache_file:
                buff.write(chunk)

        self.assertEqual(DATA, buff.getvalue())
        # verify now cache hit count for above image is 1
        self.assertEqual(1, self.driver.get_hit_count(images['public']))

    def test_open_for_read_with_exception(self):
        self.start_server(enable_cache=True)
        self.driver = centralized_db.Driver()
        self.driver.configure()
        images = self.load_data()

        self.assertFalse(self.driver.is_cached(images['public']))
        # Cache one image
        path = '/v2/cache/%s' % images['public']
        self.api_put(path)
        self.wait_for_caching(images['public'])
        # verify image is cached
        self.assertTrue(self.driver.is_cached(images['public']))
        # verify cache hit count for above image is 0
        self.assertEqual(0, self.driver.get_hit_count(images['public']))
        # Open image for read
        buff = io.BytesIO()
        try:
            with self.driver.open_for_read(images['public']):
                raise IOError
        except Exception as e:
            self.assertIsInstance(e, IOError)

        self.assertEqual(b'', buff.getvalue())
        # verify now cache hit count for above image is 1 even exception is
        # raised
        self.assertEqual(1, self.driver.get_hit_count(images['public']))
